@@ -1,24 +1,16 @@
 /*
- * $Id$
- *
  * debug print 
  *
- *
  * Copyright (C) 2001-2003 FhG Fokus
  *
- * This file is part of ser, a free SIP server.
+ * This file is part of Kamailio, a free SIP server.
  *
- * ser is free software; you can redistribute it and/or modify
+ * Kamailio is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version
  *
- * For a license to use the ser software under conditions
- * other than those described here, or to purchase support for this
- * software, please contact iptel.org by e-mail at the following addresses:
- *    info@iptel.org
- *
- * ser is distributed in the hope that it will be useful,
+ * Kamailio is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -29,7 +21,7 @@
  */
 /*!
  * \file
- * \brief SIP-router core :: 
+ * \brief Kamailio core :: Debug print
  * \ingroup core
  * Module: \ref core
  */
@@ -396,7 +388,7 @@ void log_prefix_set(sip_msg_t *msg)
 {
 	if(log_prefix_pvs == NULL)
 		return;
-	if(msg==NULL) {
+	if(msg==NULL || !(IS_SIP(msg) || IS_SIP_REPLY(msg))) {
 		log_prefix_val = NULL;
 		return;
 	}
