@@ -1,23 +1,16 @@
 /*
- * $Id$
- *
  * destination set
  *
  * Copyright (C) 2001-2004 FhG FOKUS
  *
- * This file is part of ser, a free SIP server.
+ * This file is part of Kamailio, a free SIP server.
  *
- * ser is free software; you can redistribute it and/or modify
+ * Kamailio is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version
  *
- * For a license to use the ser software under conditions
- * other than those described here, or to purchase support for this
- * software, please contact iptel.org by e-mail at the following addresses:
- *    info@iptel.org
- *
- * ser is distributed in the hope that it will be useful,
+ * Kamailio is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -27,7 +20,7 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-/** destination set / branches support.
+/** Kamailio core :: destination set / branches support.
  * @file dset.c
  * @ingroup core
  * Module: @ref core
@@ -39,6 +32,7 @@
 #include "parser/parser_f.h"
 #include "parser/parse_uri.h"
 #include "parser/msg_parser.h"
+#include "globals.h"
 #include "ut.h"
 #include "hash_func.h"
 #include "error.h"
@@ -58,9 +52,9 @@
 
 /* 
  * Where we store URIs of additional transaction branches
- * (-1 because of the default branch, #0)
+ * (sr_dst_max_branches - 1 : because of the default branch for r-uri, #0 in tm)
  */
-static struct branch branches[MAX_BRANCHES - 1];
+static struct branch *branches = NULL;
 
 /* how many of them we have */
 unsigned int nr_branches = 0;
@@ -78,6 +72,23 @@ static qvalue_t ruri_q = Q_UNSPECIFIED;
 static flag_t ruri_bflags;
 
 
+int init_dst_set(void)
+{
+	if(sr_dst_max_branches<=0 || sr_dst_max_branches>=MAX_BRANCHES_LIMIT) {
+		LM_ERR("invalid value for max branches parameter: %u\n",
+				sr_dst_max_branches);
+		return -1;
+	}
+	/* sr_dst_max_branches - 1 : because of the default branch for r-uri, #0 in tm */
+	branches = (branch_t*)pkg_malloc((sr_dst_max_branches-1)*sizeof(branch_t));
+	if(branches==NULL) {
+		LM_ERR("not enough memory to initialize destination branches\n");
+		return -1;
+	}
+	memset(branches, 0, (sr_dst_max_branches-1)*sizeof(branch_t));
+	return 0;
+}
+
 /*! \brief
  * Return pointer to branch[idx] structure
  * @param idx - branch index
@@ -111,14 +122,14 @@ int drop_sip_branch(int idx)
 		return 0;
 	if(idx<0 && (int)nr_branches+idx<0)
 		return 0;
+	if(idx<0)
+		idx += nr_branches;
 	/* last branch */
 	if(idx==nr_branches-1)
 	{
 		nr_branches--;
 		return 0;
 	}
-	if(idx<0)
-		idx = nr_branches+idx;
 	/* shift back one position */
 	for(; idx<nr_branches-1; idx++)
 		memcpy(&branches[idx], &branches[idx+1], sizeof(branch_t));
@@ -333,7 +344,7 @@ int append_branch(struct sip_msg* msg, str* uri, str* dst_uri, str* path,
 	/* if we have already set up the maximum number
 	 * of branches, don't try new ones 
 	 */
-	if (unlikely(nr_branches == MAX_BRANCHES - 1)) {
+	if (unlikely(nr_branches == sr_dst_max_branches - 1)) {
 		LM_ERR("max nr of branches exceeded\n");
 		ser_error = E_TOO_MANY_BRANCHES;
 		return -1;
@@ -341,6 +352,11 @@ int append_branch(struct sip_msg* msg, str* uri, str* dst_uri, str* path,
 
 	/* if not parameterized, take current uri */
 	if (uri==0 || uri->len==0 || uri->s==0) {
+		if(msg==NULL) {
+			LM_ERR("no new uri and no msg to take r-uri\n");
+			ser_error = E_INVALID_PARAMS;
+			return -1;
+		}
 		if (msg->new_uri.s)
 			luri = msg->new_uri;
 		else
