@@ -1,6 +1,4 @@
-/**
- * $Id$
- *
+/*
  * Copyright (C) 2013 Konstantin Mosesov
  *
  * This file is part of Kamailio, a free SIP server.
@@ -33,7 +31,7 @@
 
 #include "global.h"
 #include "utils.h"
-#include "java_mod.h"
+#include "app_java_mod.h"
 #include "java_iface.h"
 #include "java_support.h"
 #include "java_native_methods.h"
