@@ -836,7 +836,7 @@ int peer_connect(peer *p)
 	int sock;
 	unsigned int option = 1;
 
-	struct addrinfo *ainfo=0,*res=0,hints;
+	struct addrinfo *ainfo=0,*res=0,*sainfo=0,hints;
 	char buf[256],host[256],serv[256];
 	int error;
 
@@ -870,6 +870,25 @@ int peer_connect(peer *p)
 			continue;
 		}
 
+		/* try to set the local socket used to connect to the peer */
+		if (p->src_addr.s && p->src_addr.len > 0) {
+			LM_DBG("peer_connect(): connetting to peer via src addr=%.*s",p->src_addr.len, p->src_addr.s);
+			memset (&hints, 0, sizeof(hints));
+			hints.ai_flags = AI_NUMERICHOST;
+			hints.ai_socktype = SOCK_STREAM;
+			error = getaddrinfo(p->src_addr.s, NULL, &hints, &sainfo);
+
+			if (error!=0){
+				LM_WARN("peer_connect(): error getting client socket on %.*s:%s\n",
+					p->src_addr.len,p->src_addr.s,gai_strerror(error));
+			} else {
+				if (bind(sock, sainfo->ai_addr, sainfo->ai_addrlen )) {
+					LM_WARN("peer_connect(): error opening client socket on %.*s:%s\n",
+						p->src_addr.len,p->src_addr.s,strerror(errno));
+				}
+			}
+		}
+
 		{// Connect with timeout
 			int x;
 			x=fcntl(sock,F_GETFL,0);
