@@ -26,11 +26,15 @@
  */
 
 #include "km_pg_con.h"
+#include "pg_mod.h"
 #include "../../mem/mem.h"
 #include "../../dprint.h"
 #include "../../ut.h"
+#include "../../tls_hooks_init.h" 
 #include <string.h>
 #include <time.h>
+#include <netinet/in.h>
+#include <netinet/tcp.h>
 
 
 /*!
@@ -45,6 +49,9 @@ struct pg_con* db_postgres_new_connection(struct db_id* id)
 {
 	struct pg_con* ptr;
 	char *ports;
+	int i = 0;
+	const char *keywords[10], *values[10];
+	char to[16];
 
 	LM_DBG("db_id = %p\n", id);
  
@@ -66,6 +73,8 @@ struct pg_con* db_postgres_new_connection(struct db_id* id)
 
 	if (id->port) {
 		ports = int2str(id->port, 0);
+		keywords[i] = "port";
+		values[i++] = ports;
 		LM_DBG("opening connection: postgres://xxxx:xxxx@%s:%d/%s\n", ZSW(id->host),
 			id->port, ZSW(id->database));
 	} else {
@@ -74,8 +83,27 @@ struct pg_con* db_postgres_new_connection(struct db_id* id)
 			ZSW(id->database));
 	}
 
- 	ptr->con = PQsetdbLogin(id->host, ports, NULL, NULL, id->database, id->username, id->password);
-	LM_DBG("PQsetdbLogin(%p)\n", ptr->con);
+	keywords[i] = "host";
+	values[i++] = id->host;
+	keywords[i] = "dbname";
+	values[i++] = id->database;
+	keywords[i] = "user";
+	values[i++] = id->username;
+	keywords[i] = "password";
+	values[i++] = id->password;
+	if (pg_timeout > 0) {
+		snprintf(to, sizeof(to)-1, "%d", pg_timeout + 3);
+		keywords[i] = "connect_timeout";
+		values[i++] = to;
+	}
+
+	keywords[i] = values[i] = NULL;
+
+	/* don't attempt to re-init openssl if done already */
+	if(tls_loaded()) PQinitSSL(0);
+
+	ptr->con = PQconnectdbParams(keywords, values, 1);
+	LM_DBG("PQconnectdbParams(%p)\n", ptr->con);
 
 	if( (ptr->con == 0) || (PQstatus(ptr->con) != CONNECTION_OK) )
 	{
@@ -88,6 +116,14 @@ struct pg_con* db_postgres_new_connection(struct db_id* id)
 	ptr->timestamp = time(0);
 	ptr->id = id;
 
+#if defined(SO_KEEPALIVE) && defined(TCP_KEEPIDLE)
+	if (pg_keepalive) {
+		i = 1;
+		setsockopt(PQsocket(ptr->con), SOL_SOCKET, SO_KEEPALIVE, &i, sizeof(i));
+		setsockopt(PQsocket(ptr->con), IPPROTO_TCP, TCP_KEEPIDLE, &pg_keepalive, sizeof(pg_keepalive));
+	}
+#endif
+
 	return ptr;
 
  err:
