@@ -18,27 +18,6 @@
  * along with this program; if not, write to the Free Software 
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- * History:
- * --------
- * 2006-04-14  initial version (bogdan)
- * 2007-03-06  syncronized state machine added for dialog state. New tranzition
- *             design based on events; removed num_1xx and num_2xx (bogdan)
- * 2007-04-30  added dialog matching without DID (dialog ID), but based only
- *             on RFC3261 elements - based on an original patch submitted 
- *             by Michel Bensoussan <michel@extricom.com> (bogdan)
- * 2007-07-06  additional information stored in order to save it in the db:
- *             cseq, route_set, contact and sock_info for both caller and 
- *             callee (ancuta)
- * 2007-07-10  Optimized dlg_match_mode 2 (DID_NONE), it now employs a proper
- *             hash table lookup and isn't dependant on the is_direction 
- *             function (which requires an RR param like dlg_match_mode 0 
- *             anyways.. ;) ; based on a patch from 
- *             Tavis Paquette <tavis@galaxytelecom.net> 
- *             and Peter Baer <pbaer@galaxytelecom.net>  (bogdan)
- * 2008-04-17  added new type of callback to be triggered right before the
- *              dialog is destroyed (deleted from memory) (bogdan)
- * 2008-04-17  added new dialog flag to avoid state tranzitions from DELETED to
- *             CONFIRMED_NA due delayed "200 OK" (bogdan)
  */
 
 
@@ -63,9 +42,6 @@
 #include "dlg_req_within.h"
 #include "dlg_db_handler.h"
 
-#define MAX_LDG_LOCKS  2048
-#define MIN_LDG_LOCKS  2
-
 extern int dlg_ka_interval;
 
 /*! global dialog table */
@@ -190,8 +166,9 @@ int dlg_ka_run(ticks_t ti)
 		if(*dlg_ka_list_head == *dlg_ka_list_tail) {
 			*dlg_ka_list_head = NULL;
 			*dlg_ka_list_tail = NULL;
+		} else {
+			*dlg_ka_list_head = dka->next;
 		}
-		*dlg_ka_list_head = dka->next;
 		lock_release(dlg_ka_list_lock);
 
 		/* send keep-alive for dka */
@@ -200,10 +177,16 @@ int dlg_ka_run(ticks_t ti)
 			shm_free(dka);
 			dka = NULL;
 		} else {
-			if(dka->iflags & DLG_IFLAG_KA_SRC)
-				dlg_send_ka(dlg, DLG_CALLER_LEG, 0);
-			if(dka->iflags & DLG_IFLAG_KA_DST)
-				dlg_send_ka(dlg, DLG_CALLEE_LEG, 0);
+			if((dka->iflags & DLG_IFLAG_KA_SRC)
+					&& (dlg->state==DLG_STATE_CONFIRMED))
+				dlg_send_ka(dlg, DLG_CALLER_LEG);
+			if((dka->iflags & DLG_IFLAG_KA_DST)
+					&& (dlg->state==DLG_STATE_CONFIRMED))
+				dlg_send_ka(dlg, DLG_CALLEE_LEG);
+			if(dlg->state==DLG_STATE_DELETED) {
+				shm_free(dka);
+				dka = NULL;
+			}
 			dlg_release(dlg);
 		}
 		/* append to tail */
@@ -237,7 +220,7 @@ int dlg_clean_run(ticks_t ti)
 	tm = (unsigned int)time(NULL);
 	for(i=0; i<d_table->size; i++)
 	{
-		lock_set_get(d_table->locks, d_table->entries[i].lock_idx);
+		dlg_lock(d_table, &d_table->entries[i]);
 		dlg = d_table->entries[i].first;
 		while (dlg) {
 			tdlg = dlg;
@@ -257,7 +240,7 @@ int dlg_clean_run(ticks_t ti)
 				tdlg->dflags |= DLG_FLAG_CHANGED;
 			}
 		}
-		lock_set_release(d_table->locks, d_table->entries[i].lock_idx);
+		dlg_unlock(d_table, &d_table->entries[i]);
 	}
 	return 0;
 }
@@ -269,7 +252,6 @@ int dlg_clean_run(ticks_t ti)
  */
 int init_dlg_table(unsigned int size)
 {
-	unsigned int n;
 	unsigned int i;
 
 	dlg_ka_list_head = (dlg_ka_t **)shm_malloc(sizeof(dlg_ka_t *));
@@ -302,30 +284,13 @@ int init_dlg_table(unsigned int size)
 	d_table->size = size;
 	d_table->entries = (struct dlg_entry*)(d_table+1);
 
-	n = (size<MAX_LDG_LOCKS)?size:MAX_LDG_LOCKS;
-	for(  ; n>=MIN_LDG_LOCKS ; n-- ) {
-		d_table->locks = lock_set_alloc(n);
-		if (d_table->locks==0)
-			continue;
-		if (lock_set_init(d_table->locks)==0) {
-			lock_set_dealloc(d_table->locks);
-			d_table->locks = 0;
-			continue;
-		}
-		d_table->locks_no = n;
-		break;
-	}
-
-	if (d_table->locks==0) {
-		LM_ERR("unable to allocted at least %d locks for the hash table\n",
-			MIN_LDG_LOCKS);
-		goto error1;
-	}
-
 	for( i=0 ; i<size; i++ ) {
 		memset( &(d_table->entries[i]), 0, sizeof(struct dlg_entry) );
+		if(lock_init(&d_table->entries[i].lock)<0) {
+			LM_ERR("failed to init lock for slot: %d\n", i);
+			goto error1;
+		}
 		d_table->entries[i].next_id = rand() % (3*size);
-		d_table->entries[i].lock_idx = i % d_table->locks_no;
 	}
 
 	return 0;
@@ -425,11 +390,6 @@ void destroy_dlg_table(void)
 	if (d_table==0)
 		return;
 
-	if (d_table->locks) {
-		lock_set_destroy(d_table->locks);
-		lock_set_dealloc(d_table->locks);
-	}
-
 	for( i=0 ; i<d_table->size; i++ ) {
 		dlg = d_table->entries[i].first;
 		while (dlg) {
@@ -437,7 +397,7 @@ void destroy_dlg_table(void)
 			dlg = dlg->next;
 			destroy_dlg(l_dlg);
 		}
-
+		lock_destroy(&d_table->entries[i].lock);
 	}
 
 	shm_free(d_table);
@@ -710,7 +670,7 @@ static inline struct dlg_cell* internal_get_dlg(unsigned int h_entry,
 		if (match_dialog( dlg, callid, ftag, ttag, dir)==1) {
 			ref_dlg_unsafe(dlg, 1);
 			dlg_unlock( d_table, d_entry);
-			LM_DBG("dialog callid='%.*s' found\n on entry %u, dir=%d\n",
+			LM_DBG("dialog callid='%.*s' found on entry %u, dir=%d\n",
 				callid->len, callid->s,h_entry,*dir);
 			return dlg;
 		}
@@ -1097,12 +1057,14 @@ int dlg_set_toroute(struct dlg_cell *dlg, str *route)
 
 int	update_dlg_timeout(dlg_cell_t *dlg, int timeout)
 {
-	if(update_dlg_timer(&dlg->tl, timeout) < 0) {
-		LM_ERR("failed to update dialog lifetime\n");
-		dlg_release(dlg);
-		return -1;
-	} 
-
+	if(dlg->state!=DLG_STATE_UNCONFIRMED
+			&& dlg->state!=DLG_STATE_EARLY) {
+		if(update_dlg_timer(&dlg->tl, timeout) < 0) {
+			LM_ERR("failed to update dialog lifetime\n");
+			dlg_release(dlg);
+			return -1;
+		}
+	}
 	dlg->lifetime = timeout;
 	dlg->dflags |= DLG_FLAG_CHANGED;
 
