@@ -17,10 +17,6 @@
  * along with this program; if not, write to the Free Software 
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- * History:
- * --------
- * 2008-04-20  initial version (bogdan)
- *
  */
 
 
@@ -353,6 +349,7 @@ void remove_expired_remote_profiles(time_t te)
 						lh->next = lh->prev = NULL;
 						if(lh->linker) shm_free(lh->linker);
 						p_entry->content--;
+						lock_release(&profile->lock);
 						return;
 					}
 					lh = kh;
@@ -374,39 +371,60 @@ int remove_profile(dlg_profile_table_t *profile, str *value, str *puid)
 	unsigned int hash;
 	struct dlg_profile_entry *p_entry;
 	struct dlg_profile_hash *lh;
-	struct dlg_profile_hash *kh;
 
 	hash = calc_hash_profile(value, puid, profile);
 	lock_get(&profile->lock );
 	p_entry = &profile->entries[hash];
 	lh = p_entry->first;
-	while(lh) {
-		kh = lh->next;
-		if(lh->dlg==NULL && lh->puid_len==puid->len
-				&& lh->value.len==value->len
-				&& strncmp(lh->puid, puid->s, puid->len)==0
-				&& strncmp(lh->value.s, value->s, value->len)==0) {
-			/* last element on the list? */
-			if (lh==lh->next) {
-				p_entry->first = NULL;
-			} else {
-				if (p_entry->first==lh)
-					p_entry->first = lh->next;
-				lh->next->prev = lh->prev;
-				lh->prev->next = lh->next;
+	if(lh) {
+		do {
+			if(lh->dlg==NULL && lh->puid_len==puid->len
+					&& lh->value.len==value->len
+					&& strncmp(lh->puid, puid->s, puid->len)==0
+					&& strncmp(lh->value.s, value->s, value->len)==0) {
+				/* last element on the list? */
+				if (lh==lh->next) {
+					p_entry->first = NULL;
+				} else {
+					if (p_entry->first==lh)
+						p_entry->first = lh->next;
+					lh->next->prev = lh->prev;
+					lh->prev->next = lh->next;
+				}
+				lh->next = lh->prev = NULL;
+				if(lh->linker) shm_free(lh->linker);
+				p_entry->content--;
+				return 1;
 			}
-			lh->next = lh->prev = NULL;
-			if(lh->linker) shm_free(lh->linker);
-			p_entry->content--;
-			return 1;
-		}
-		lh = kh;
+			lh = lh->next;
+		} while(lh != p_entry->first);
 	}
 	lock_release(&profile->lock );
 	return 0;
 }
 
 
+/*!
+ * \brief Callback for cleanup of profile local vars
+ * \param msg SIP message
+ * \param flags unused
+ * \param param unused
+ * \return 1
+ */
+int cb_profile_reset( struct sip_msg *msg, unsigned int flags, void *param )
+{
+	current_dlg_msg_id = 0;
+	current_dlg_msg_pid = 0;
+	if (current_pending_linkers) {
+		destroy_linkers(current_pending_linkers);
+		current_pending_linkers = NULL;
+	}
+
+	/* need to return non-zero - 0 will break the exec of the request */
+	return 1;
+}
+
+
 /*!
  * \brief Cleanup a profile
  * \param msg SIP message
