@@ -1,6 +1,4 @@
 /*
- * $Id$
- *
  * Copyright (C) 2007 Voice System SRL
  *
  * This file is part of Kamailio, a free SIP server.
@@ -19,12 +17,16 @@
  * along with this program; if not, write to the Free Software 
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- * History:
- * --------
- * 2007-07-10  initial version (ancuta)
- * 2008-04-04  added direction reporting in dlg callbacks (bogdan)
 */
 
+
+/*!
+ * \file
+ * \brief Requests
+ * \ingroup dialog
+ * Module: \ref dialog
+ */
+
 #include <stdlib.h>
 #include <string.h>
 
@@ -50,7 +52,7 @@
 #define MAX_FWD_HDR_LEN    (sizeof(MAX_FWD_HDR) - 1)
 
 extern str dlg_extra_hdrs;
-
+extern str dlg_lreq_callee_headers;
 
 
 int free_tm_dlg(dlg_t *td)
@@ -214,7 +216,7 @@ void bye_reply_cb(struct cell* t, int type, struct tmcb_params* ps){
 			unref++;
 		}
 		/* dialog terminated (BYE) */
-		run_dlg_callbacks( DLGCB_TERMINATED, dlg, ps->req, ps->rpl, DLG_DIR_NONE, 0);
+		run_dlg_callbacks( DLGCB_TERMINATED_CONFIRMED, dlg, ps->req, ps->rpl, DLG_DIR_NONE, 0);
 
 		LM_DBG("first final reply\n");
 		/* derefering the dialog */
@@ -262,6 +264,10 @@ void dlg_ka_cb(struct cell* t, int type, struct tmcb_params* ps){
 	}
 
 	if(ps->code==408 || ps->code==481) {
+		if (dlg->state != DLG_STATE_CONFIRMED) {
+			LM_DBG("skip updating non-confirmed dialogs\n");
+			goto done;
+		}
 		if(update_dlg_timer(&dlg->tl, 10)<0) {
 			LM_ERR("failed to update dialog lifetime\n");
 			goto done;
@@ -280,12 +286,19 @@ static inline int build_extra_hdr(struct dlg_cell * cell, str *extra_hdrs,
 		str *str_hdr)
 {
 	char *p;
+	int blen;
 
 	str_hdr->len = MAX_FWD_HDR_LEN + dlg_extra_hdrs.len;
 	if(extra_hdrs && extra_hdrs->len>0)
 		str_hdr->len += extra_hdrs->len;
 
-	str_hdr->s = (char*)pkg_malloc( str_hdr->len * sizeof(char) );
+	blen = str_hdr->len + 1 /* '\0' */;
+
+	/* reserve space for callee headers in local requests */
+	if(dlg_lreq_callee_headers.len>0)
+		blen += dlg_lreq_callee_headers.len + 2 /* '\r\n' */;
+
+	str_hdr->s = (char*)pkg_malloc( blen * sizeof(char) );
 	if(!str_hdr->s){
 		LM_ERR("out of pkg memory\n");
 		goto error;
@@ -320,6 +333,7 @@ static inline int send_bye(struct dlg_cell * cell, int dir, str *hdrs)
 	str met = {"BYE", 3};
 	int result;
 	dlg_iuid_t *iuid = NULL;
+	str lhdrs;
 
 	/* do not send BYE request for non-confirmed dialogs (not supported) */
 	if (cell->state != DLG_STATE_CONFIRMED_NA && cell->state != DLG_STATE_CONFIRMED) {
@@ -343,7 +357,20 @@ static inline int send_bye(struct dlg_cell * cell, int dir, str *hdrs)
 		goto err;
 	}
 
-	set_uac_req(&uac_r, &met, hdrs, NULL, dialog_info, TMCB_LOCAL_COMPLETED,
+	lhdrs = *hdrs;
+
+	if(dir==DLG_CALLEE_LEG && dlg_lreq_callee_headers.len>0) {
+		/* space allocated in hdrs->s by build_extra_hdrs() */
+		memcpy(lhdrs.s+lhdrs.len, dlg_lreq_callee_headers.s,
+				dlg_lreq_callee_headers.len);
+		lhdrs.len += dlg_lreq_callee_headers.len;
+		if(dlg_lreq_callee_headers.s[dlg_lreq_callee_headers.len-1]!='\n') {
+			strncpy(lhdrs.s+lhdrs.len, CRLF, CRLF_LEN);
+			lhdrs.len += CRLF_LEN;
+		}
+	}
+
+	set_uac_req(&uac_r, &met, &lhdrs, NULL, dialog_info, TMCB_LOCAL_COMPLETED,
 				bye_reply_cb, (void*)iuid);
 	result = d_tmb.t_request_within(&uac_r);
 
@@ -370,7 +397,7 @@ err:
  * 		DLG_CALLER_LEG (0): caller
  * 		DLG_CALLEE_LEG (1): callee
  */
-int dlg_send_ka(dlg_cell_t *dlg, int dir, str *hdrs)
+int dlg_send_ka(dlg_cell_t *dlg, int dir)
 {
 	uac_req_t uac_r;
 	dlg_t* di;
@@ -406,8 +433,13 @@ int dlg_send_ka(dlg_cell_t *dlg, int dir, str *hdrs)
 		goto err;
 	}
 
-	set_uac_req(&uac_r, &met, hdrs, NULL, di, TMCB_LOCAL_COMPLETED,
+	if(dir==DLG_CALLEE_LEG && dlg_lreq_callee_headers.len>0) {
+		set_uac_req(&uac_r, &met, &dlg_lreq_callee_headers, NULL, di,
+				TMCB_LOCAL_COMPLETED, dlg_ka_cb, (void*)iuid);
+	} else {
+		set_uac_req(&uac_r, &met, NULL, NULL, di, TMCB_LOCAL_COMPLETED,
 				dlg_ka_cb, (void*)iuid);
+	}
 	result = d_tmb.t_request_within(&uac_r);
 
 	if(result < 0){
@@ -521,6 +553,9 @@ int dlg_bye_all(struct dlg_cell *dlg, str *hdrs)
 	str all_hdrs = { 0, 0 };
 	int ret;
 
+	/* run dialog terminated callbacks */
+	run_dlg_callbacks( DLGCB_TERMINATED, dlg, NULL, NULL, DLG_DIR_NONE, 0);
+
 	if ((build_extra_hdr(dlg, hdrs, &all_hdrs)) != 0)
 	{
 		LM_ERR("failed to build dlg headers\n");
