@@ -1,5 +1,4 @@
 /**
- * $Id$
  *
  * Copyright (C) 2008 Elena-Ramona Modroiu (asipto.com)
  *
@@ -140,6 +139,37 @@ void keyvalue_destroy(keyvalue_t *res)
 	memset(res, 0, sizeof(keyvalue_t));
 }
 
+/**
+ * recursive/re-entrant lock of the slot in hash table
+ */
+void ht_slot_lock(ht_t *ht, int idx)
+{
+	int mypid;
+
+	mypid = my_pid();
+	if (likely(atomic_get(&ht->entries[idx].locker_pid) != mypid)) {
+		lock_get(&ht->entries[idx].lock);
+		atomic_set(&ht->entries[idx].locker_pid, mypid);
+	} else {
+		/* locked within the same process that executed us */
+		ht->entries[idx].rec_lock_level++;
+	}
+}
+
+/**
+ * recursive/re-entrant unlock of the slot in hash table
+ */
+void ht_slot_unlock(ht_t *ht, int idx)
+{
+	if (likely(ht->entries[idx].rec_lock_level == 0)) {
+		atomic_set(&ht->entries[idx].locker_pid, 0);
+		lock_release(&ht->entries[idx].lock);
+	} else  {
+		/* recursive locked => decrease lock count */
+		ht->entries[idx].rec_lock_level--;
+	}
+}
+
 ht_cell_t* ht_cell_new(str *name, int type, int_str *val, unsigned int cellid)
 {
 	ht_cell_t *cell;
@@ -393,7 +423,7 @@ int ht_set_cell(ht_t *ht, str *name, int type, int_str *val, int mode)
 	if(ht->htexpire>0)
 		now = time(NULL);
 	prev = NULL;
-	if(mode) lock_get(&ht->entries[idx].lock);
+	if(mode) ht_slot_lock(ht, idx);
 	it = ht->entries[idx].first;
 	while(it!=NULL && it->cellid < hid)
 	{
@@ -425,7 +455,7 @@ int ht_set_cell(ht_t *ht, str *name, int type, int_str *val, int mode)
 						if(cell == NULL)
 						{
 							LM_ERR("cannot create new cell\n");
-							if(mode) lock_release(&ht->entries[idx].lock);
+							if(mode) ht_slot_unlock(ht, idx);
 							return -1;
 						}
 						cell->next = it->next;
@@ -446,7 +476,7 @@ int ht_set_cell(ht_t *ht, str *name, int type, int_str *val, int mode)
 					if(ht->updateexpire)
 						it->expire = now + ht->htexpire;
 				}
-				if(mode) lock_release(&ht->entries[idx].lock);
+				if(mode) ht_slot_unlock(ht, idx);
 				return 0;
 			} else {
 				if(type&AVP_VAL_STR)
@@ -456,7 +486,7 @@ int ht_set_cell(ht_t *ht, str *name, int type, int_str *val, int mode)
 					if(cell == NULL)
 					{
 						LM_ERR("cannot create new cell.\n");
-						if(mode) lock_release(&ht->entries[idx].lock);
+						if(mode) ht_slot_unlock(ht, idx);
 						return -1;
 					}
 					cell->expire = now + ht->htexpire;
@@ -475,7 +505,7 @@ int ht_set_cell(ht_t *ht, str *name, int type, int_str *val, int mode)
 					if(ht->updateexpire)
 						it->expire = now + ht->htexpire;
 				}
-				if(mode) lock_release(&ht->entries[idx].lock);
+				if(mode) ht_slot_unlock(ht, idx);
 				return 0;
 			}
 		}
@@ -487,7 +517,7 @@ int ht_set_cell(ht_t *ht, str *name, int type, int_str *val, int mode)
 	if(cell == NULL)
 	{
 		LM_ERR("cannot create new cell.\n");
-		if(mode) lock_release(&ht->entries[idx].lock);
+		if(mode) ht_slot_unlock(ht, idx);
 		return -1;
 	}
 	cell->expire = now + ht->htexpire;
@@ -507,7 +537,7 @@ int ht_set_cell(ht_t *ht, str *name, int type, int_str *val, int mode)
 		prev->next = cell;
 	}
 	ht->entries[idx].esize++;
-	if(mode) lock_release(&ht->entries[idx].lock);
+	if(mode) ht_slot_unlock(ht, idx);
 	return 0;
 }
 
@@ -528,7 +558,7 @@ int ht_del_cell(ht_t *ht, str *name)
 	if(ht->entries[idx].first==NULL)
 		return 0;
 	
-	lock_get(&ht->entries[idx].lock);
+	ht_slot_lock(ht, idx);
 	it = ht->entries[idx].first;
 	while(it!=NULL && it->cellid < hid)
 		it = it->next;
@@ -545,13 +575,13 @@ int ht_del_cell(ht_t *ht, str *name)
 			if(it->next)
 				it->next->prev = it->prev;
 			ht->entries[idx].esize--;
-			lock_release(&ht->entries[idx].lock);
+			ht_slot_unlock(ht, idx);
 			ht_cell_free(it);
 			return 0;
 		}
 		it = it->next;
 	}
-	lock_release(&ht->entries[idx].lock);
+	ht_slot_unlock(ht, idx);
 	return 0;
 }
 
@@ -575,7 +605,7 @@ ht_cell_t* ht_cell_value_add(ht_t *ht, str *name, int val, int mode,
 	if(ht->htexpire>0)
 		now = time(NULL);
 	prev = NULL;
-	if(mode) lock_get(&ht->entries[idx].lock);
+	if(mode) ht_slot_lock(ht, idx);
 	it = ht->entries[idx].first;
 	while(it!=NULL && it->cellid < hid)
 	{
@@ -606,7 +636,7 @@ ht_cell_t* ht_cell_value_add(ht_t *ht, str *name, int val, int mode,
 					if(it->next)
 						it->next->prev = it->prev;
 					ht->entries[idx].esize--;
-					lock_release(&ht->entries[idx].lock);
+					if(mode) ht_slot_unlock(ht, idx);
 					ht_cell_free(it);
 					return NULL;
 				}
@@ -615,7 +645,7 @@ ht_cell_t* ht_cell_value_add(ht_t *ht, str *name, int val, int mode,
 			if(it->flags&AVP_VAL_STR)
 			{
 				/* string value cannot be incremented */
-				if(mode) lock_release(&ht->entries[idx].lock);
+				if(mode) ht_slot_unlock(ht, idx);
 				return NULL;
 			} else {
 				it->value.n += val;
@@ -625,7 +655,7 @@ ht_cell_t* ht_cell_value_add(ht_t *ht, str *name, int val, int mode,
 					if(old->msize>=it->msize)
 					{
 						memcpy(old, it, it->msize);
-						lock_release(&ht->entries[idx].lock);
+						if(mode) ht_slot_unlock(ht, idx);
 						return old;
 					}
 				}
@@ -633,7 +663,7 @@ ht_cell_t* ht_cell_value_add(ht_t *ht, str *name, int val, int mode,
 				if(cell!=NULL)
 					memcpy(cell, it, it->msize);
 
-				if(mode) lock_release(&ht->entries[idx].lock);
+				if(mode) ht_slot_unlock(ht, idx);
 				return cell;
 			}
 		}
@@ -642,13 +672,16 @@ ht_cell_t* ht_cell_value_add(ht_t *ht, str *name, int val, int mode,
 	}
 	/* add val if htable has an integer init value */
 	if(ht->flags!=PV_VAL_INT)
+	{
+		if(mode) ht_slot_unlock(ht, idx);
 		return NULL;
+	}
 	isval.n = ht->initval.n + val;
 	it = ht_cell_new(name, 0, &isval, hid);
 	if(it == NULL)
 	{
 		LM_ERR("cannot create new cell.\n");
-		if(mode) lock_release(&ht->entries[idx].lock);
+		if(mode) ht_slot_unlock(ht, idx);
 		return NULL;
 	}
 	it->expire = now + ht->htexpire;
@@ -673,7 +706,7 @@ ht_cell_t* ht_cell_value_add(ht_t *ht, str *name, int val, int mode,
 		if(old->msize>=it->msize)
 		{
 			memcpy(old, it, it->msize);
-			lock_release(&ht->entries[idx].lock);
+			if(mode) ht_slot_unlock(ht, idx);
 			return old;
 		}
 	}
@@ -681,7 +714,7 @@ ht_cell_t* ht_cell_value_add(ht_t *ht, str *name, int val, int mode,
 	if(cell!=NULL)
 		memcpy(cell, it, it->msize);
 
-	if(mode) lock_release(&ht->entries[idx].lock);
+	if(mode) ht_slot_unlock(ht, idx);
 	return cell;
 }
 
@@ -703,7 +736,7 @@ ht_cell_t* ht_cell_pkg_copy(ht_t *ht, str *name, ht_cell_t *old)
 	if(ht->entries[idx].first==NULL)
 		return NULL;
 	
-	lock_get(&ht->entries[idx].lock);
+	ht_slot_lock(ht, idx);
 	it = ht->entries[idx].first;
 	while(it!=NULL && it->cellid < hid)
 		it = it->next;
@@ -724,7 +757,7 @@ ht_cell_t* ht_cell_pkg_copy(ht_t *ht, str *name, ht_cell_t *old)
 				if(it->next)
 					it->next->prev = it->prev;
 				ht->entries[idx].esize--;
-				lock_release(&ht->entries[idx].lock);
+				ht_slot_unlock(ht, idx);
 				ht_cell_free(it);
 				return NULL;
 			}
@@ -733,19 +766,19 @@ ht_cell_t* ht_cell_pkg_copy(ht_t *ht, str *name, ht_cell_t *old)
 				if(old->msize>=it->msize)
 				{
 					memcpy(old, it, it->msize);
-					lock_release(&ht->entries[idx].lock);
+					ht_slot_unlock(ht, idx);
 					return old;
 				}
 			}
 			cell = (ht_cell_t*)pkg_malloc(it->msize);
 			if(cell!=NULL)
 				memcpy(cell, it, it->msize);
-			lock_release(&ht->entries[idx].lock);
+			ht_slot_unlock(ht, idx);
 			return cell;
 		}
 		it = it->next;
 	}
-	lock_release(&ht->entries[idx].lock);
+	ht_slot_unlock(ht, idx);
 	return NULL;
 }
 
@@ -762,7 +795,7 @@ int ht_dbg(void)
 				ht->name.s, ht->htid, ht->htexpire);
 		for(i=0; i<ht->htsize; i++)
 		{
-			lock_get(&ht->entries[i].lock);
+			ht_slot_lock(ht, i);
 			LM_ERR("htable[%d] -- <%d>\n", i, ht->entries[i].esize);
 			it = ht->entries[i].first;
 			while(it)
@@ -776,7 +809,7 @@ int ht_dbg(void)
 					LM_ERR("\tv-i:%d\n", it->value.n);
 				it = it->next;
 			}
-			lock_release(&ht->entries[i].lock);
+			ht_slot_unlock(ht, i);
 		}
 		ht = ht->next;
 	}
@@ -945,7 +978,7 @@ void ht_timer(unsigned int ticks, void *param)
 			for(i=0; i<ht->htsize; i++)
 			{
 				/* free entries */
-				lock_get(&ht->entries[i].lock);
+				ht_slot_lock(ht, i);
 				it = ht->entries[i].first;
 				while(it)
 				{
@@ -965,7 +998,7 @@ void ht_timer(unsigned int ticks, void *param)
 					}
 					it = it0;
 				}
-				lock_release(&ht->entries[i].lock);
+				ht_slot_unlock(ht, i);
 			}
 		}
 		ht = ht->next;
@@ -1040,7 +1073,7 @@ int ht_set_cell_expire(ht_t *ht, str *name, int type, int_str *val)
 	LM_DBG("set auto-expire to %u (%d)\n", (unsigned int)now,
 			val->n);
 
-	lock_get(&ht->entries[idx].lock);
+	ht_slot_lock(ht, idx);
 	it = ht->entries[idx].first;
 	while(it!=NULL && it->cellid < hid)
 		it = it->next;
@@ -1051,12 +1084,12 @@ int ht_set_cell_expire(ht_t *ht, str *name, int type, int_str *val)
 		{
 			/* update value */
 			it->expire = now;
-			lock_release(&ht->entries[idx].lock);
+			ht_slot_unlock(ht, idx);
 			return 0;
 		}
 		it = it->next;
 	}
-	lock_release(&ht->entries[idx].lock);
+	ht_slot_unlock(ht, idx);
 	return 0;
 }
 
@@ -1080,7 +1113,7 @@ int ht_get_cell_expire(ht_t *ht, str *name, unsigned int *val)
 	idx = ht_get_entry(hid, ht->htsize);
 
 	now = time(NULL);
-	lock_get(&ht->entries[idx].lock);
+	ht_slot_lock(ht, idx);
 	it = ht->entries[idx].first;
 	while(it!=NULL && it->cellid < hid)
 		it = it->next;
@@ -1091,12 +1124,12 @@ int ht_get_cell_expire(ht_t *ht, str *name, unsigned int *val)
 		{
 			/* update value */
 			*val = (unsigned int)(it->expire - now);
-			lock_release(&ht->entries[idx].lock);
+			ht_slot_unlock(ht, idx);
 			return 0;
 		}
 		it = it->next;
 	}
-	lock_release(&ht->entries[idx].lock);
+	ht_slot_unlock(ht, idx);
 	return 0;
 }
 
@@ -1121,7 +1154,7 @@ int ht_rm_cell_re(str *sre, ht_t *ht, int mode)
 	for(i=0; i<ht->htsize; i++)
 	{
 		/* free entries */
-		lock_get(&ht->entries[i].lock);
+		ht_slot_lock(ht, i);
 		it = ht->entries[i].first;
 		while(it)
 		{
@@ -1149,7 +1182,7 @@ int ht_rm_cell_re(str *sre, ht_t *ht, int mode)
 			}
 			it = it0;
 		}
-		lock_release(&ht->entries[i].lock);
+		ht_slot_unlock(ht, i);
 	}
 	regfree(&re);
 	return 0;
@@ -1167,7 +1200,7 @@ int ht_reset_content(ht_t *ht)
 	for(i=0; i<ht->htsize; i++)
 	{
 		/* free entries */
-		lock_get(&ht->entries[i].lock);
+		ht_slot_lock(ht, i);
 		it = ht->entries[i].first;
 		while(it)
 		{
@@ -1182,7 +1215,7 @@ int ht_reset_content(ht_t *ht)
 			ht_cell_free(it);
 			it = it0;
 		}
-		lock_release(&ht->entries[i].lock);
+		ht_slot_unlock(ht, i);
 	}
 	return 0;
 }
@@ -1285,7 +1318,7 @@ int ht_count_cells_re(str *sre, ht_t *ht, int mode)
 	for(i=0; i<ht->htsize; i++)
 	{
 		/* free entries */
-		lock_get(&ht->entries[i].lock);
+		ht_slot_lock(ht, i);
 		it = ht->entries[i].first;
 		while(it)
 		{
@@ -1332,7 +1365,7 @@ int ht_count_cells_re(str *sre, ht_t *ht, int mode)
 			}
 			it = it0;
 		}
-		lock_release(&ht->entries[i].lock);
+		ht_slot_unlock(ht, i);
 	}
 	if(op==1)
 		regfree(&re);
@@ -1388,7 +1421,7 @@ int ht_iterator_start(str *iname, str *hname)
 		{
 			if(_ht_iterators[k].slot>=0 && _ht_iterators[k].slot<_ht_iterators[k].ht->htsize)
 			{
-				lock_release(&_ht_iterators[k].ht->entries[_ht_iterators[k].slot].lock);
+				ht_slot_unlock(_ht_iterators[k].ht, _ht_iterators[k].slot);
 			}
 		}
 	} else {
@@ -1400,6 +1433,8 @@ int ht_iterator_start(str *iname, str *hname)
 		}
 		strncpy(_ht_iterators[k].bname, iname->s, iname->len);
 		_ht_iterators[k].bname[iname->len] = '\0';
+		_ht_iterators[k].name.len = iname->len;
+		_ht_iterators[k].name.s = _ht_iterators[k].bname;
 	}
 	_ht_iterators[k].it = NULL;
 	_ht_iterators[k].slot = 0;
@@ -1455,19 +1490,19 @@ int ht_iterator_next(str *iname)
 		}
 		/* next is not in the same slot - release and try next one */
 		_ht_iterators[k].it = NULL;
-		lock_release(&_ht_iterators[k].ht->entries[_ht_iterators[k].slot].lock);
+		ht_slot_unlock(_ht_iterators[k].ht, _ht_iterators[k].slot);
 		_ht_iterators[k].slot++;
 	}
 
 	for( ; _ht_iterators[k].slot<_ht_iterators[k].ht->htsize; _ht_iterators[k].slot++)
 	{
-		lock_get(&_ht_iterators[k].ht->entries[_ht_iterators[k].slot].lock);
+		ht_slot_lock(_ht_iterators[k].ht, _ht_iterators[k].slot);
 		if(_ht_iterators[k].ht->entries[_ht_iterators[k].slot].first!=NULL)
 		{
 			_ht_iterators[k].it = _ht_iterators[k].ht->entries[_ht_iterators[k].slot].first;
 			return 0;
 		}
-		lock_release(&_ht_iterators[k].ht->entries[_ht_iterators[k].slot].lock);
+		ht_slot_unlock(_ht_iterators[k].ht, _ht_iterators[k].slot);
 	}
 	return -1;
 }
@@ -1487,7 +1522,7 @@ int ht_iterator_end(str *iname)
 				{
 					if(_ht_iterators[i].slot>=0 && _ht_iterators[i].slot<_ht_iterators[i].ht->htsize)
 					{
-						lock_release(&_ht_iterators[i].ht->entries[_ht_iterators[i].slot].lock);
+						ht_slot_unlock(_ht_iterators[i].ht, _ht_iterators[i].slot);
 					}
 				}
 				memset(&_ht_iterators[i], 0, sizeof(ht_iterator_t));
