@@ -4,7 +4,7 @@
 #include <amqp.h>
 #include <amqp_framing.h>
 #include <amqp_tcp_socket.h>
-#include <json/json.h>
+#include <json.h>
 #include <uuid/uuid.h>
 #include "../../mem/mem.h"
 #include "../../timer_proc.h"
@@ -15,12 +15,13 @@
 
 
 #include "kz_amqp.h"
+#include "kz_json.h"
 
 #define RET_AMQP_ERROR 2
 
-
-kz_amqp_conn_pool_ptr kz_pool = NULL;
+kz_amqp_connection_pool_ptr kz_pool = NULL;
 kz_amqp_bindings_ptr kz_bindings = NULL;
+int bindings_count = 0;
 
 static unsigned long rpl_query_routing_key_count = 0;
 
@@ -34,6 +35,7 @@ extern struct timeval kz_sock_tv;
 extern struct timeval kz_amqp_tv;
 extern struct timeval kz_qtimeout_tv;
 extern struct timeval kz_ack_tv;
+extern struct timeval kz_timer_tv;
 
 extern int dbk_internal_loop_count;
 extern int dbk_consumer_loop_count;
@@ -43,6 +45,11 @@ extern int dbk_consumer_ack_loop_count;
 extern int dbk_single_consumer_on_reconnect;
 extern int dbk_consume_messages_on_reconnect;
 
+extern pv_spec_t kz_query_timeout_spec;
+
+const amqp_bytes_t kz_amqp_empty_bytes = { 0, NULL };
+const amqp_table_t kz_amqp_empty_table = { 0, NULL };
+
 static char *kz_amqp_str_dup(str *src)
 {
 	char *res;
@@ -146,7 +153,7 @@ void kz_amqp_free_bind(kz_amqp_bind_ptr bind)
 	shm_free(bind);
 }
 
-void kz_amqp_free_connection(kz_amqp_conn_ptr conn)
+void kz_amqp_free_connection(kz_amqp_connection_ptr conn)
 {
 	if(!conn)
 		return;
@@ -271,12 +278,48 @@ kz_amqp_bind_ptr kz_amqp_bind_alloc(str* exchange, str* exchange_type, str* queu
 
 void kz_amqp_init_connection_pool() {
 	if(kz_pool == NULL) {
-		kz_pool = (kz_amqp_conn_pool_ptr) shm_malloc(sizeof(kz_amqp_conn_pool));
-		memset(kz_pool, 0, sizeof(kz_amqp_conn_pool));
+		kz_pool = (kz_amqp_connection_pool_ptr) shm_malloc(sizeof(kz_amqp_connection_pool));
+		memset(kz_pool, 0, sizeof(kz_amqp_connection_pool));
 	}
 }
 
-void kz_amqp_init() {
+int kz_amqp_bind_init_targeted_channel(int idx )
+{
+    kz_amqp_bind_ptr bind = NULL;
+    str rpl_exch = str_init("targeted");
+    str rpl_exch_type = str_init("direct");
+    int ret = -1;
+    char serverid[512];
+
+    bind = (kz_amqp_bind_ptr)shm_malloc(sizeof(kz_amqp_bind));
+	if(bind == NULL) {
+		LM_ERR("error allocation memory for reply\n");
+		goto error;
+	}
+	memset(bind, 0, sizeof(kz_amqp_bind));
+
+	bind->exchange = kz_amqp_bytes_dup_from_str(&rpl_exch);
+	bind->exchange_type = kz_amqp_bytes_dup_from_str(&rpl_exch_type);
+
+    sprintf(serverid, "kamailio@%.*s-<%d-%d>", dbk_node_hostname.len, dbk_node_hostname.s, my_pid(), idx);
+    bind->queue = kz_amqp_bytes_dup_from_string(serverid);
+
+    sprintf(serverid, "kamailio@%.*s-<%d>-targeted-%d", dbk_node_hostname.len, dbk_node_hostname.s, my_pid(), idx);
+    bind->routing_key = kz_amqp_bytes_dup_from_string(serverid);
+
+    if (bind->exchange.bytes == NULL || bind->routing_key.bytes == NULL || bind->queue.bytes == NULL) {
+		LM_ERR("Out of memory allocating for exchange/routing_key\n");
+		goto error;
+    }
+
+    channels[idx].targeted = bind;
+    return 0;
+ error:
+	kz_amqp_free_bind(bind);
+    return ret;
+}
+
+int kz_amqp_init() {
 	int i;
 	kz_amqp_init_connection_pool();
 	if(kz_bindings == NULL) {
@@ -288,8 +331,17 @@ void kz_amqp_init() {
 		memset(channels, 0, dbk_channels * sizeof(kz_amqp_channel));
 		for(i=0; i < dbk_channels; i++) {
 			channels[i].channel = i+1;
+			if(lock_init(&channels[i].lock)==NULL) {
+				LM_ERR("could not initialize locks for channels\n");
+				return 0;
+			}
+			if(kz_amqp_bind_init_targeted_channel(i)) {
+				LM_ERR("could not initialize targeted channels\n");
+				return 0;
+			}
 		}
 	}
+	return 1;
 }
 
 void kz_amqp_destroy() {
@@ -318,9 +370,9 @@ void kz_amqp_destroy() {
 	}
 
 	if(kz_pool != NULL) {
-		kz_amqp_conn_ptr conn = kz_pool->head;
+		kz_amqp_connection_ptr conn = kz_pool->head;
 		while(conn != NULL) {
-			kz_amqp_conn_ptr tofree = conn;
+			kz_amqp_connection_ptr tofree = conn;
 			conn = conn->next;
 			kz_amqp_free_connection(tofree);
 		}
@@ -335,7 +387,7 @@ static char* KZ_URL_ROOT = "/";
 
 int kz_amqp_add_connection(modparam_t type, void* val)
 {
-	kz_amqp_init_connection_pool(); // find a better way
+	kz_amqp_init_connection_pool();
 
 	char* url = (char*) val;
 	int len = strlen(url);
@@ -344,8 +396,8 @@ int kz_amqp_add_connection(modparam_t type, void* val)
 		return -1;
 	}
 
-	kz_amqp_conn_ptr newConn = shm_malloc(sizeof(kz_amqp_conn));
-	memset(newConn, 0, sizeof(kz_amqp_conn));
+	kz_amqp_connection_ptr newConn = shm_malloc(sizeof(kz_amqp_connection));
+	memset(newConn, 0, sizeof(kz_amqp_connection));
 
 	newConn->url = shm_malloc( (KZ_URL_MAX_SIZE + 1) * sizeof(char) );
 	memset(newConn->url, 0, (KZ_URL_MAX_SIZE + 1) * sizeof(char));
@@ -425,19 +477,19 @@ int kz_amqp_connection_open(kz_amqp_conn_ptr rmq) {
     	goto error;
     }
 
-    if (amqp_socket_open(rmq->socket, rmq->info.host, rmq->info.port)) {
+    if (amqp_socket_open(rmq->socket, rmq->info->info.host, rmq->info->info.port)) {
     	LM_DBG("Failed to open TCP socket to AMQP broker\n");
     	goto error;
     }
 
     if (kz_amqp_error("Logging in", amqp_login(rmq->conn,
-					   rmq->info.vhost,
+					   rmq->info->info.vhost,
 					   0,
 					   131072,
 					   0,
 					   AMQP_SASL_METHOD_PLAIN,
-					   rmq->info.user,
-					   rmq->info.password))) {
+					   rmq->info->info.user,
+					   rmq->info->info.password))) {
 
     	LM_ERR("Login to AMQP broker failed!\n");
     	goto error;
@@ -466,6 +518,9 @@ int kz_amqp_channel_open(kz_amqp_conn_ptr rmq, amqp_channel_t channel) {
 }
 
 kz_amqp_conn_ptr kz_amqp_get_connection() {
+	return NULL;
+
+	/*
 	kz_amqp_conn_ptr ptr = NULL;
 	if(kz_pool == NULL) {
 		return NULL;
@@ -492,9 +547,12 @@ kz_amqp_conn_ptr kz_amqp_get_connection() {
 //	lock_release(&kz_pool->lock);
 
    	return ptr;
+   	*/
 }
 
 kz_amqp_conn_ptr kz_amqp_get_next_connection() {
+	return NULL;
+	/*
 	kz_amqp_conn_ptr ptr = NULL;
 	if(kz_pool == NULL) {
 		return NULL;
@@ -518,8 +576,46 @@ kz_amqp_conn_ptr kz_amqp_get_next_connection() {
 
 
    	return ptr;
+   	*/
 }
 
+int kz_amqp_open_next_connection(kz_amqp_conn_ptr ptr) {
+	if(ptr == NULL) {
+		LM_ERR("OPEN CONNECTION PTR == NULL\n");
+		return -1;
+	}
+
+	if(kz_pool == NULL) {
+		LM_ERR("OPEN CONNECTION POOL == NULL\n");
+		return -2;
+	}
+
+	if(ptr->info == NULL) {
+		ptr->info = kz_pool->head;
+	} else {
+		ptr->info = ptr->info->next;
+		if(ptr->info == NULL) {
+			ptr->info = kz_pool->head;
+		}
+	}
+
+	while(ptr->conn == NULL) {
+		if(kz_amqp_connection_open(ptr) == 0) {
+			break;
+		}
+		ptr->info = ptr->info->next;
+		if(ptr->info == NULL) {
+			LM_INFO("all connections tried, restarting from head\n");
+			sleep(3);
+			ptr->info = kz_pool->head;
+		}
+	}
+
+
+   	return 0;
+}
+
+
 int kz_amqp_consume_error(amqp_connection_state_t conn)
 {
 	amqp_frame_t frame;
@@ -599,8 +695,6 @@ void kz_amqp_add_payload_common_properties(json_obj_ptr json_obj, char* server_i
     sprintf(node_name, "kamailio@%.*s", dbk_node_hostname.len, dbk_node_hostname.s);
     json_object_object_add(json_obj, BLF_JSON_NODE,
 			   json_object_new_string(node_name));
-//    json_object_object_add(json_obj, BLF_JSON_SERVERID,
-//			   json_object_new_string(server_id));
     json_object_object_add(json_obj, BLF_JSON_MSG_ID,
 			   json_object_new_string_len(unique->s, unique->len));
 
@@ -627,13 +721,9 @@ int kz_amqp_pipe_send(str *str_exchange, str *str_routing_key, str *str_payload)
 
 
     /* parse json  and add extra fields */
-    json_obj = json_tokener_parse(str_payload->s);
-    if (is_error(json_obj))
-    {
-		LM_ERR("Error parsing json: %s\n",json_tokener_errors[-(unsigned long)json_obj]);
-		LM_ERR("%s\n", str_payload->s);
-		goto error;
-    }
+    json_obj = kz_json_parse(str_payload->s);
+    if (json_obj == NULL)
+	goto error;
 
     kz_amqp_add_payload_common_properties(json_obj, serverid, &unique_string);
 
@@ -679,7 +769,7 @@ int kz_amqp_pipe_send(str *str_exchange, str *str_routing_key, str *str_payload)
 	return ret;
 }
 
-int kz_amqp_pipe_send_receive(str *str_exchange, str *str_routing_key, str *str_payload, json_obj_ptr* json_ret )
+int kz_amqp_pipe_send_receive(str *str_exchange, str *str_routing_key, str *str_payload, struct timeval* kz_timeout, json_obj_ptr* json_ret )
 {
 	int ret = 1;
     json_obj_ptr json_obj = NULL;
@@ -701,13 +791,9 @@ int kz_amqp_pipe_send_receive(str *str_exchange, str *str_routing_key, str *str_
 
 
     /* parse json  and add extra fields */
-    json_obj = json_tokener_parse(str_payload->s);
-    if (is_error(json_obj))
-    {
-		LM_ERR("Error parsing json: %s\n",json_tokener_errors[-(unsigned long)json_obj]);
-		LM_ERR("%s\n", str_payload->s);
-		goto error;
-    }
+    json_obj = kz_json_parse(str_payload->s);
+    if (json_obj == NULL)
+	goto error;
 
     kz_amqp_add_payload_common_properties(json_obj, serverid, &unique_string);
 
@@ -723,7 +809,9 @@ int kz_amqp_pipe_send_receive(str *str_exchange, str *str_routing_key, str *str_
 	cmd->routing_key = kz_amqp_str_dup(str_routing_key);
 	cmd->reply_routing_key = kz_amqp_string_dup(serverid);
 	cmd->payload = kz_amqp_string_dup(payload);
-	cmd->timeout = kz_qtimeout_tv;
+
+	cmd->timeout = *kz_timeout;
+
 	if(cmd->payload == NULL || cmd->routing_key == NULL || cmd->exchange == NULL) {
 		LM_ERR("failed to allocate kz_amqp_cmd parameters in process %d\n", getpid());
 		goto error;
@@ -743,13 +831,9 @@ int kz_amqp_pipe_send_receive(str *str_exchange, str *str_routing_key, str *str_
 		lock_get(&cmd->lock);
 		switch(cmd->return_code) {
 		case AMQP_RESPONSE_NORMAL:
-			json_body = json_tokener_parse(cmd->return_payload);
-		    if (is_error(json_body))
-		    {
-				LM_ERR("Error parsing body json: %s\n",json_tokener_errors[-(unsigned long)json_body]);
-				LM_ERR("JSON : %s\n", cmd->return_payload);
-				goto error;
-		    }
+			json_body = kz_json_parse(cmd->return_payload);
+		    if (json_body == NULL)
+			goto error;
 		    *json_ret = json_body;
 		    ret = 0;
 		    break;
@@ -819,6 +903,7 @@ int kz_amqp_query_ex(struct sip_msg* msg, char* exchange, char* routing_key, cha
 	  str json_s;
 	  str exchange_s;
 	  str routing_key_s;
+	  struct timeval kz_timeout = kz_qtimeout_tv;
 
 	  if(last_payload_result)
 		pkg_free(last_payload_result);
@@ -849,8 +934,19 @@ int kz_amqp_query_ex(struct sip_msg* msg, char* exchange, char* routing_key, cha
 
 		json_object_put(j);
 
+		if(kz_query_timeout_spec.type != PVT_NONE) {
+			pv_value_t pv_val;
+			if(pv_get_spec_value( msg, &kz_query_timeout_spec, &pv_val) == 0) {
+				if((pv_val.flags & PV_VAL_INT) && pv_val.ri != 0 ) {
+					kz_timeout.tv_usec = (pv_val.ri % 1000) * 1000;
+					kz_timeout.tv_sec = pv_val.ri / 1000;
+					LM_DBG("setting timeout to %i,%i\n", (int) kz_timeout.tv_sec, (int) kz_timeout.tv_usec);
+				}
+			}
+		}
+
 		json_obj_ptr ret = NULL;
-		int res = kz_amqp_pipe_send_receive(&exchange_s, &routing_key_s, &json_s, &ret );
+		int res = kz_amqp_pipe_send_receive(&exchange_s, &routing_key_s, &json_s, &kz_timeout, &ret );
 
 		if(res != 0) {
 			return -1;
@@ -944,6 +1040,7 @@ int kz_amqp_subscribe_simple(struct sip_msg* msg, char* exchange, char* exchange
 
 	kz_bindings->tail = binding;
 	binding->bind = bind;
+	bindings_count++;
 
     return 1;
 
@@ -979,13 +1076,10 @@ int kz_amqp_subscribe(struct sip_msg* msg, char* payload)
 		return -1;
 	}
 
-    json_obj = json_tokener_parse(payload_s.s);
-    if (is_error(json_obj))
-    {
-		LM_ERR("Error parsing json: %s\n",json_tokener_errors[-(unsigned long)json_obj]);
-		LM_ERR("%s\n", payload_s.s);
-		return -1;
-    }
+    json_obj = kz_json_parse(payload_s.s);
+    if (json_obj == NULL)
+	return -1;
+    
 
     json_extract_field("exchange", exchange_s);
     json_extract_field("type", exchange_type_s);
@@ -994,32 +1088,32 @@ int kz_amqp_subscribe(struct sip_msg* msg, char* payload)
     json_extract_field("event_key", key_s);
     json_extract_field("event_subkey", subkey_s);
 
-    tmpObj = json_object_object_get(json_obj, "passive");
+    tmpObj = kz_json_get_object(json_obj, "passive");
     if(tmpObj != NULL) {
     	passive = json_object_get_int(tmpObj);
     }
 
-    tmpObj = json_object_object_get(json_obj, "durable");
+    tmpObj = kz_json_get_object(json_obj, "durable");
     if(tmpObj != NULL) {
     	durable = json_object_get_int(tmpObj);
     }
 
-    tmpObj = json_object_object_get(json_obj, "exclusive");
+    tmpObj = kz_json_get_object(json_obj, "exclusive");
     if(tmpObj != NULL) {
     	exclusive = json_object_get_int(tmpObj);
     }
 
-    tmpObj = json_object_object_get(json_obj, "auto_delete");
+    tmpObj = kz_json_get_object(json_obj, "auto_delete");
     if(tmpObj != NULL) {
     	auto_delete = json_object_get_int(tmpObj);
     }
 
-    tmpObj = json_object_object_get(json_obj, "no_ack");
+    tmpObj = kz_json_get_object(json_obj, "no_ack");
     if(tmpObj != NULL) {
     	no_ack = json_object_get_int(tmpObj);
     }
 
-    tmpObj = json_object_object_get(json_obj, "wait_for_consumer_ack");
+    tmpObj = kz_json_get_object(json_obj, "wait_for_consumer_ack");
     if(tmpObj != NULL) {
     	wait_for_consumer_ack = json_object_get_int(tmpObj);
     }
@@ -1054,6 +1148,7 @@ int kz_amqp_subscribe(struct sip_msg* msg, char* payload)
 
 	kz_bindings->tail = binding;
 	binding->bind = bind;
+	bindings_count++;
 
     if(json_obj != NULL)
        	json_object_put(json_obj);
@@ -1156,14 +1251,50 @@ int get_channel_index() {
 			return n;
 		}
 	if(channel_index == 0) {
-		LM_ERR("max channels (%d) reached. please exit kazoo and change db_kazoo amqp_max_channels param", dbk_channels);
+		LM_ERR("max channels (%d) reached. please exit kamailio and change kazoo amqp_max_channels param", dbk_channels);
 		return -1;
 	}
 	channel_index = 0;
 	return get_channel_index();
 }
 
-int kz_amqp_bind_targeted_channel(kz_amqp_conn_ptr kz_conn, int loopcount, int idx )
+int kz_amqp_bind_targeted_channel(kz_amqp_conn_ptr kz_conn, int idx )
+{
+    kz_amqp_bind_ptr bind = channels[idx].targeted;
+    amqp_queue_declare_ok_t *r = NULL;
+    int ret = -1;
+
+    r = amqp_queue_declare(kz_conn->conn, channels[idx].channel, bind->queue, 0, 0, 1, 1, kz_amqp_empty_table);
+    if (kz_amqp_error("Declaring queue", amqp_get_rpc_reply(kz_conn->conn)))
+    {
+		goto error;
+    }
+
+	amqp_exchange_declare(kz_conn->conn, channels[idx].channel, bind->exchange, bind->exchange_type, 0, 0, kz_amqp_empty_table);
+    if (kz_amqp_error("Declaring exchange", amqp_get_rpc_reply(kz_conn->conn)))
+    {
+		ret = -RET_AMQP_ERROR;
+		goto error;
+    }
+
+    if (amqp_queue_bind(kz_conn->conn, channels[idx].channel, bind->queue, bind->exchange, bind->routing_key, kz_amqp_empty_table) < 0
+	    || kz_amqp_error("Binding queue", amqp_get_rpc_reply(kz_conn->conn)))
+    {
+		goto error;
+    }
+
+    if (amqp_basic_consume(kz_conn->conn, channels[idx].channel, bind->queue, kz_amqp_empty_bytes, 0, 1, 1, kz_amqp_empty_table) < 0
+	    || kz_amqp_error("Consuming", amqp_get_rpc_reply(kz_conn->conn)))
+    {
+		goto error;
+    }
+
+    return 0;
+ error:
+    return ret;
+}
+
+int kz_amqp_bind_targeted_channel_ex(kz_amqp_conn_ptr kz_conn, int loopcount, int idx )
 {
     kz_amqp_bind_ptr bind = NULL;
     amqp_queue_declare_ok_t *r = NULL;
@@ -1194,26 +1325,26 @@ int kz_amqp_bind_targeted_channel(kz_amqp_conn_ptr kz_conn, int loopcount, int i
 		goto error;
     }
 
-    r = amqp_queue_declare(kz_conn->conn, channels[idx].channel, bind->queue, 0, 0, 1, 1, amqp_empty_table);
+    r = amqp_queue_declare(kz_conn->conn, channels[idx].channel, bind->queue, 0, 0, 1, 1, kz_amqp_empty_table);
     if (kz_amqp_error("Declaring queue", amqp_get_rpc_reply(kz_conn->conn)))
     {
 		goto error;
     }
 
-	amqp_exchange_declare(kz_conn->conn, channels[idx].channel, bind->exchange, bind->exchange_type, 0, 0, amqp_empty_table);
+	amqp_exchange_declare(kz_conn->conn, channels[idx].channel, bind->exchange, bind->exchange_type, 0, 0, kz_amqp_empty_table);
     if (kz_amqp_error("Declaring exchange", amqp_get_rpc_reply(kz_conn->conn)))
     {
 		ret = -RET_AMQP_ERROR;
 		goto error;
     }
 
-    if (amqp_queue_bind(kz_conn->conn, channels[idx].channel, bind->queue, bind->exchange, bind->routing_key, amqp_empty_table) < 0
+    if (amqp_queue_bind(kz_conn->conn, channels[idx].channel, bind->queue, bind->exchange, bind->routing_key, kz_amqp_empty_table) < 0
 	    || kz_amqp_error("Binding queue", amqp_get_rpc_reply(kz_conn->conn)))
     {
 		goto error;
     }
 
-    if (amqp_basic_consume(kz_conn->conn, channels[idx].channel, bind->queue, amqp_empty_bytes, 0, 1, 1, amqp_empty_table) < 0
+    if (amqp_basic_consume(kz_conn->conn, channels[idx].channel, bind->queue, kz_amqp_empty_bytes, 0, 1, 1, kz_amqp_empty_table) < 0
 	    || kz_amqp_error("Consuming", amqp_get_rpc_reply(kz_conn->conn)))
     {
 		goto error;
@@ -1230,13 +1361,55 @@ int kz_amqp_bind_targeted_channels(kz_amqp_conn_ptr kz_conn , int loopcount)
 {
 	int i, ret;
 	for(i = 0; i < dbk_channels; i++) {
-		ret = kz_amqp_bind_targeted_channel(kz_conn, loopcount, i);
+		ret = kz_amqp_bind_targeted_channel_ex(kz_conn, loopcount, i);
 		if(ret != 0)
 			return ret;
 	}
 	return 0;
 }
 
+int kz_amqp_bind_consumer_ex(kz_amqp_conn_ptr kz_conn, kz_amqp_bind_ptr bind, int idx, kz_amqp_channel_ptr chan)
+{
+    int ret = -1;
+
+    amqp_queue_declare(kz_conn->conn, chan[idx].channel, bind->queue, bind->passive, bind->durable, bind->exclusive, bind->auto_delete, kz_amqp_empty_table);
+    if (kz_amqp_error("Declaring queue", amqp_get_rpc_reply(kz_conn->conn)))
+    {
+		ret = -RET_AMQP_ERROR;
+		goto error;
+    }
+
+	amqp_exchange_declare(kz_conn->conn, chan[idx].channel, bind->exchange, bind->exchange_type, 0, 0, kz_amqp_empty_table);
+    if (kz_amqp_error("Declaring exchange", amqp_get_rpc_reply(kz_conn->conn)))
+    {
+		ret = -RET_AMQP_ERROR;
+		goto error;
+    }
+
+    LM_DBG("QUEUE BIND\n");
+    if (amqp_queue_bind(kz_conn->conn, chan[idx].channel, bind->queue, bind->exchange, bind->routing_key, kz_amqp_empty_table) < 0
+	    || kz_amqp_error("Binding queue", amqp_get_rpc_reply(kz_conn->conn)))
+    {
+		ret = -RET_AMQP_ERROR;
+		goto error;
+    }
+
+    LM_DBG("BASIC CONSUME\n");
+    if (amqp_basic_consume(kz_conn->conn, chan[idx].channel, bind->queue, kz_amqp_empty_bytes, 0, bind->no_ack, 0, kz_amqp_empty_table) < 0
+	    || kz_amqp_error("Consuming", amqp_get_rpc_reply(kz_conn->conn)))
+    {
+		ret = -RET_AMQP_ERROR;
+		goto error;
+    }
+
+    chan[idx].state = KZ_AMQP_CONSUMING;
+	chan[idx].consumer = bind;
+    ret = idx;
+ error:
+
+    return ret;
+}
+
 
 int kz_amqp_bind_consumer(kz_amqp_conn_ptr kz_conn, kz_amqp_bind_ptr bind)
 {
@@ -1244,14 +1417,14 @@ int kz_amqp_bind_consumer(kz_amqp_conn_ptr kz_conn, kz_amqp_bind_ptr bind)
 
     int	idx = get_channel_index();
 
-    amqp_queue_declare(kz_conn->conn, channels[idx].channel, bind->queue, bind->passive, bind->durable, bind->exclusive, bind->auto_delete, amqp_empty_table);
+    amqp_queue_declare(kz_conn->conn, channels[idx].channel, bind->queue, bind->passive, bind->durable, bind->exclusive, bind->auto_delete, kz_amqp_empty_table);
     if (kz_amqp_error("Declaring queue", amqp_get_rpc_reply(kz_conn->conn)))
     {
 		ret = -RET_AMQP_ERROR;
 		goto error;
     }
 
-	amqp_exchange_declare(kz_conn->conn, channels[idx].channel, bind->exchange, bind->exchange_type, 0, 0, amqp_empty_table);
+	amqp_exchange_declare(kz_conn->conn, channels[idx].channel, bind->exchange, bind->exchange_type, 0, 0, kz_amqp_empty_table);
     if (kz_amqp_error("Declaring exchange", amqp_get_rpc_reply(kz_conn->conn)))
     {
 		ret = -RET_AMQP_ERROR;
@@ -1259,7 +1432,7 @@ int kz_amqp_bind_consumer(kz_amqp_conn_ptr kz_conn, kz_amqp_bind_ptr bind)
     }
 
     LM_DBG("QUEUE BIND\n");
-    if (amqp_queue_bind(kz_conn->conn, channels[idx].channel, bind->queue, bind->exchange, bind->routing_key, amqp_empty_table) < 0
+    if (amqp_queue_bind(kz_conn->conn, channels[idx].channel, bind->queue, bind->exchange, bind->routing_key, kz_amqp_empty_table) < 0
 	    || kz_amqp_error("Binding queue", amqp_get_rpc_reply(kz_conn->conn)))
     {
 		ret = -RET_AMQP_ERROR;
@@ -1267,7 +1440,7 @@ int kz_amqp_bind_consumer(kz_amqp_conn_ptr kz_conn, kz_amqp_bind_ptr bind)
     }
 
     LM_DBG("BASIC CONSUME\n");
-    if (amqp_basic_consume(kz_conn->conn, channels[idx].channel, bind->queue, amqp_empty_bytes, 0, bind->no_ack, 0, amqp_empty_table) < 0
+    if (amqp_basic_consume(kz_conn->conn, channels[idx].channel, bind->queue, kz_amqp_empty_bytes, 0, bind->no_ack, 0, kz_amqp_empty_table) < 0
 	    || kz_amqp_error("Consuming", amqp_get_rpc_reply(kz_conn->conn)))
     {
 		ret = -RET_AMQP_ERROR;
@@ -1282,13 +1455,12 @@ int kz_amqp_bind_consumer(kz_amqp_conn_ptr kz_conn, kz_amqp_bind_ptr bind)
     return ret;
 }
 
-
 int kz_amqp_send_ex(kz_amqp_conn_ptr kz_conn, kz_amqp_cmd_ptr cmd, kz_amqp_channel_state state, int idx)
 {
 	amqp_bytes_t exchange;
 	amqp_bytes_t routing_key;
 	amqp_bytes_t payload;
-	int ret = 1;
+	int ret = -1;
     json_obj_ptr json_obj = NULL;
 
 	amqp_basic_properties_t props;
@@ -1308,28 +1480,29 @@ int kz_amqp_send_ex(kz_amqp_conn_ptr kz_conn, kz_amqp_cmd_ptr cmd, kz_amqp_chann
     routing_key = amqp_bytes_malloc_dup(amqp_cstring_bytes(cmd->routing_key));
     payload = amqp_bytes_malloc_dup(amqp_cstring_bytes(cmd->payload));
 
-    json_obj = json_tokener_parse(cmd->payload);
-    if (is_error(json_obj))
-    {
-		LM_ERR("Error parsing json: %s\n",json_tokener_errors[-(unsigned long)json_obj]);
-		LM_ERR("%s\n", cmd->payload);
-		goto error;
-    }
+    json_obj = kz_json_parse(cmd->payload);
+    if (json_obj == NULL)
+    	goto error;
 
-    if(json_object_object_get(json_obj, BLF_JSON_SERVERID) == NULL) {
+    if(kz_json_get_object(json_obj, BLF_JSON_SERVERID) == NULL) {
         json_object_object_add(json_obj, BLF_JSON_SERVERID, json_object_new_string((char*)channels[idx].targeted->routing_key.bytes));
     	amqp_bytes_free(payload);
         payload = amqp_bytes_malloc_dup(amqp_cstring_bytes((char*)json_object_to_json_string(json_obj)));
     }
 
-
-	amqp_basic_publish(kz_conn->conn, channels[idx].channel, exchange, routing_key, 0, 0, &props, payload);
+	int amqpres = amqp_basic_publish(kz_conn->conn, channels[idx].channel, exchange, routing_key, 0, 0, &props, payload);
+	if ( amqpres != AMQP_STATUS_OK ) {
+		LM_ERR("Failed to publish\n");
+		ret = -1;
+		goto error;
+	}
 
 	if ( kz_amqp_error("Publishing",  amqp_get_rpc_reply(kz_conn->conn)) ) {
 		LM_ERR("Failed to publish\n");
 		ret = -1;
 		goto error;
 	}
+	gettimeofday(&channels[idx].timer, NULL);
 	channels[idx].state = state;
 	channels[idx].cmd = cmd;
 
@@ -1412,13 +1585,9 @@ void kz_amqp_consumer_event(int child_no, char *payload, char* event_key, char*
 
     eventData = payload;
 
-    json_obj = json_tokener_parse(payload);
-    if (is_error(json_obj))
-    {
-		LM_ERR("Error parsing json: %s\n",json_tokener_errors[-(unsigned long)json_obj]);
-		LM_ERR("%s\n", payload);
+    json_obj = kz_json_parse(payload);
+    if (json_obj == NULL)
 		return;
-    }
 
     char* key = (event_key == NULL ? dbk_consumer_event_key.s : event_key);
     char* subkey = (event_subkey == NULL ? dbk_consumer_event_subkey.s : event_subkey);
@@ -1462,7 +1631,7 @@ void kz_amqp_consumer_loop(int child_no)
 	LM_DBG("starting consumer %d\n", child_no);
 	close(kz_pipe_fds[child_no*2+1]);
 	int data_pipe = kz_pipe_fds[child_no*2];
-	int back_idx = (dbk_consumer_processes+1)*2+1;
+//	int back_idx = (dbk_consumer_processes+1)*2+1;
 
 	fd_set fdset;
     int selret;
@@ -1482,6 +1651,7 @@ void kz_amqp_consumer_loop(int child_no)
 				if(read(data_pipe, &ptr, sizeof(ptr)) == sizeof(ptr)) {
 					LM_DBG("consumer %d received payload %s\n", child_no, ptr->payload);
 					kz_amqp_consumer_event(child_no, ptr->payload, ptr->event_key, ptr->event_subkey);
+					/*
 					if(ptr->channel > 0 && ptr->delivery_tag > 0) {
 						kz_amqp_cmd_ptr cmd = kz_amqp_alloc_pipe_cmd();
 						cmd->type = KZ_AMQP_ACK;
@@ -1491,6 +1661,7 @@ void kz_amqp_consumer_loop(int child_no)
 							LM_ERR("failed to send ack to AMQP Manager in process %d, write to command pipe: %s\n", getpid(), strerror(errno));
 						}
 					}
+					*/
 					kz_amqp_free_consumer_delivery(ptr);
 				}
 			}
@@ -1525,7 +1696,7 @@ void kz_amqp_send_consumer_event_ex(char* payload, char* event_key, char* event_
 	ptr->payload = payload;
 	ptr->event_key = event_key;
 	ptr->event_subkey = event_subkey;
-	if (write(kz_pipe_fds[consumer*2+1], &ptr, sizeof(ptr)) != sizeof(ptr)) {
+	if (write(kz_pipe_fds[(consumer+2)*2+1], &ptr, sizeof(ptr)) != sizeof(ptr)) {
 		LM_ERR("failed to send payload to consumer %d : %s\nPayload %s\n", consumer, strerror(errno), payload);
 	}
 
@@ -1591,7 +1762,7 @@ void kz_amqp_manager_loop(int child_no)
     		sleep(3);
     	}
 
-    	kz_amqp_fire_connection_event("open", kzconn->info.host);
+    	kz_amqp_fire_connection_event("open", kzconn->info->info.host);
 
     	loopcount++;
     	for(i=0,channel_res=0; i < dbk_channels && channel_res == 0; i++) {
@@ -1611,7 +1782,7 @@ void kz_amqp_manager_loop(int child_no)
     		/* end cleanup */
     		channel_res = kz_amqp_channel_open(kzconn, channels[i].channel);
     		if(channel_res == 0) {
-    			kz_amqp_bind_targeted_channel(kzconn, loopcount, i);
+    			kz_amqp_bind_targeted_channel_ex(kzconn, loopcount, i);
 				channels[i].state = KZ_AMQP_FREE;
     		}
     	}
@@ -1805,6 +1976,291 @@ void kz_amqp_manager_loop(int child_no)
 			firstLoop = 0;
     	}
     	kz_amqp_connection_close(kzconn);
-    	kz_amqp_fire_connection_event("closed", kzconn->info.host);
+    	kz_amqp_fire_connection_event("closed", kzconn->info->info.host);
+    }
+}
+
+
+/* check timeouts */
+void kz_amqp_timeout_proc(int child_no)
+{
+	kz_amqp_cmd_ptr cmd;
+	int i;
+    while(1) {
+		struct timeval now;
+		usleep(kz_timer_tv.tv_usec);
+		for(i=0; i < dbk_channels; i++) {
+			gettimeofday(&now, NULL);
+			if(channels[i].state == KZ_AMQP_CALLING
+					&& channels[i].cmd != NULL
+					&& check_timeout(&now, &channels[i].timer, &channels[i].cmd->timeout)) {
+				lock_get(&channels[i].lock);
+				if(channels[i].cmd != NULL)
+				{
+					cmd = channels[i].cmd;
+					LM_DBG("Kazoo Query timeout - %s\n", cmd->payload);
+					cmd->return_code = -1;
+					lock_release(&cmd->lock);
+					channels[i].cmd = NULL;
+					channels[i].state = KZ_AMQP_FREE;
+				}
+				lock_release(&channels[i].lock);
+			}
+		}
+	}
+}
+
+void kz_amqp_publisher_proc(int child_no)
+{
+	LM_DBG("starting publisher %d\n", child_no);
+	close(kz_pipe_fds[child_no*2+1]);
+	int data_pipe = kz_pipe_fds[child_no*2];
+	LM_DBG("publisher started %d\n", child_no);
+    fd_set fdset;
+    int idx, i;
+    int selret;
+	int OK;
+    kz_amqp_conn_ptr kzconn;
+	kz_amqp_cmd_ptr cmd;
+	int channel_res;
+
+    kzconn = (kz_amqp_conn_ptr)pkg_malloc(sizeof(kz_amqp_conn));
+    if(kzconn == NULL)
+    {
+    	LM_ERR("NO MORE PACKAGE MEMORY\n");
+    	return;
+    }
+    memset(kzconn, 0, sizeof(kz_amqp_conn));
+
+    while(1) {
+    	OK = 1;
+   		if(kz_amqp_open_next_connection(kzconn)) {
+   			LM_ERR("Error opening connection\n");
+   			sleep(3);
+   			continue;
+   		}
+    	kz_amqp_fire_connection_event("open", kzconn->info->info.host);
+
+    	for(i=0,channel_res=0; i < dbk_channels && channel_res == 0; i++) {
+			/* start cleanup */
+			channels[i].state = KZ_AMQP_CLOSED;
+			cmd = channels[i].cmd;
+			if(cmd != NULL) {
+				channels[i].cmd = NULL;
+				cmd->return_code = -1;
+				lock_release(&cmd->lock);
+			}
+			/* end cleanup */
+
+			/* bind targeted channels */
+			channel_res = kz_amqp_channel_open(kzconn, channels[i].channel);
+			if(channel_res == 0) {
+				channels[i].state = KZ_AMQP_FREE;
+			}
+
+    	}
+
+    	while(OK) {
+			FD_ZERO(&fdset);
+			FD_SET(data_pipe, &fdset);
+			selret = select(FD_SETSIZE, &fdset, NULL, NULL, NULL);
+			if (selret < 0) {
+				LM_ERR("select() failed: %s\n", strerror(errno));
+				continue;
+			} else if (!selret) {
+				continue;
+			} else {
+				if(FD_ISSET(data_pipe, &fdset) && read(data_pipe, &cmd, sizeof(cmd)) == sizeof(cmd)) {
+					switch (cmd->type) {
+						case KZ_AMQP_PUBLISH:
+							idx = kz_amqp_send(kzconn, cmd);
+							if(idx >= 0) {
+								cmd->return_code = AMQP_RESPONSE_NORMAL;
+							} else {
+								cmd->return_code = -1;
+								OK = 0;
+								LM_ERR("ERROR SENDING PUBLISH");
+							}
+							channels[idx].cmd = NULL;
+							lock_release(&cmd->lock);
+							channels[idx].state = KZ_AMQP_FREE;
+							break;
+						case KZ_AMQP_CALL:
+							idx = kz_amqp_send_receive(kzconn, cmd);
+							if(idx < 0) {
+								channels[idx].cmd = NULL;
+								cmd->return_code = -1;
+								lock_release(&cmd->lock);
+								channels[idx].state = KZ_AMQP_FREE;
+								LM_ERR("ERROR SENDING QUERY");
+								OK = 0;
+							}
+							break;
+						default:
+							LM_DBG("unknown pipe cmd %d\n", cmd->type);
+							break;
+					}
+				}
+        	}
+    	}
+    	kz_amqp_connection_close(kzconn);
+    	kz_amqp_fire_connection_event("closed", kzconn->info->info.host);
+    }
+}
+
+void kz_amqp_consumer_proc(int child_no)
+{
+	LM_DBG("starting consumer %d\n", child_no);
+	close(kz_pipe_fds[child_no*2+1]);
+    int i, idx;
+	int OK;
+	char* payload;
+	int channel_res;
+    kz_amqp_conn_ptr kzconn;
+    kz_amqp_channel_ptr consumer_channels = NULL;
+
+    kzconn = (kz_amqp_conn_ptr)pkg_malloc(sizeof(kz_amqp_conn));
+    if(kzconn == NULL)
+    {
+    	LM_ERR("NO MORE PACKAGE MEMORY\n");
+    	return;
+    }
+    memset(kzconn, 0, sizeof(kz_amqp_conn));
+
+    consumer_channels = (kz_amqp_channel_ptr)pkg_malloc(sizeof(kz_amqp_channel)*bindings_count);
+    if(consumer_channels == NULL)
+    {
+    	LM_ERR("NO MORE PACKAGE MEMORY\n");
+    	return;
+    }
+	for(i=0; i < bindings_count; i++)
+		consumer_channels[i].channel = dbk_channels + i + 1;
+
+    while(1) {
+    	OK = 1;
+   		if(kz_amqp_open_next_connection(kzconn)) {
+   			LM_ERR("Error opening connection\n");
+   			sleep(3);
+   			continue;
+   		}
+    	kz_amqp_fire_connection_event("open", kzconn->info->info.host);
+
+    	/* reset channels */
+
+    	for(i=0,channel_res=0; i < dbk_channels && channel_res == 0; i++) {
+			/* start cleanup */
+			channels[i].consumer = NULL;
+
+			/*
+			if(channels[i].targeted != NULL) {
+				kz_amqp_free_bind(channels[i].targeted);
+				channels[i].targeted = NULL;
+			}
+			*/
+
+			/* end cleanup */
+
+			/* bind targeted channels */
+			channel_res = kz_amqp_channel_open(kzconn, channels[i].channel);
+			if(channel_res == 0) {
+				kz_amqp_bind_targeted_channel(kzconn, i);
+			}
+    	}
+
+    	for(i=0,channel_res=0; i < bindings_count && channel_res == 0; i++) {
+			/* start cleanup */
+    		consumer_channels[i].consumer = NULL;
+			/* end cleanup */
+
+			/* bind targeted channels */
+			channel_res = kz_amqp_channel_open(kzconn, consumer_channels[i].channel);
+    	}
+
+    	i = 0;
+		/* bind consumers */
+		if(kz_bindings != NULL) {
+			kz_amqp_binding_ptr binding = kz_bindings->head;
+			while(binding != NULL) {
+				kz_amqp_bind_consumer_ex(kzconn, binding->bind, i, consumer_channels);
+				binding = binding->next;
+				i++;
+			}
+		}
+
+		LM_DBG("CONSUMER INIT DONE\n");
+
+		while(OK) {
+			payload = NULL;
+			amqp_envelope_t envelope;
+			amqp_maybe_release_buffers(kzconn->conn);
+			amqp_rpc_reply_t reply = amqp_consume_message(kzconn->conn, &envelope, NULL, 0);
+			switch(reply.reply_type) {
+			case AMQP_RESPONSE_LIBRARY_EXCEPTION:
+				switch(reply.library_error) {
+				case AMQP_STATUS_HEARTBEAT_TIMEOUT:
+					LM_ERR("AMQP_STATUS_HEARTBEAT_TIMEOUT\n");
+					OK=0;
+					break;
+				case AMQP_STATUS_TIMEOUT:
+					break;
+				case AMQP_STATUS_UNEXPECTED_STATE:
+					LM_DBG("AMQP_STATUS_UNEXPECTED_STATE\n");
+					OK = kz_amqp_consume_error(kzconn->conn);
+					break;
+				default:
+					OK = 0;
+					break;
+				};
+				break;
+
+			case AMQP_RESPONSE_NORMAL:
+				idx = envelope.channel-1;
+				if(idx < dbk_channels) {
+					switch(channels[idx].state) {
+						case KZ_AMQP_CALLING:
+							lock_get(&channels[idx].lock);
+							if(channels[idx].cmd != NULL) {
+								channels[idx].cmd->return_payload = kz_amqp_bytes_dup(envelope.message.body);
+								channels[idx].cmd->return_code = AMQP_RESPONSE_NORMAL;
+								lock_release(&channels[idx].cmd->lock);
+								channels[idx].cmd = NULL;
+								channels[idx].state = KZ_AMQP_FREE;
+							}
+							lock_release(&channels[idx].lock);
+							break;
+						default:
+							LM_DBG("ignoring received payload on consumer - %.*s\n", (int) envelope.message.body.len, (char*)envelope.message.body.bytes);
+							break;
+					}
+				} else {
+					idx = idx - dbk_channels;
+					kz_amqp_send_consumer_event_ex(kz_amqp_bytes_dup(envelope.message.body),
+						kz_amqp_bytes_dup(consumer_channels[idx].consumer->event_key),
+						kz_amqp_bytes_dup(consumer_channels[idx].consumer->event_subkey),
+						0, 0, 1);
+					if(!consumer_channels[idx].consumer->no_ack ) {
+						if(amqp_basic_ack(kzconn->conn, envelope.channel, envelope.delivery_tag, 0 ) < 0) {
+							LM_ERR("AMQP ERROR TRYING TO ACK A MSG\n");
+							OK = 0;
+						}
+					}
+				}
+				break;
+			case AMQP_RESPONSE_SERVER_EXCEPTION:
+				LM_ERR("AMQP_RESPONSE_SERVER_EXCEPTION in consume\n");
+				OK = 0;
+				break;
+
+			default:
+				LM_ERR("UNHANDLED AMQP_RESPONSE in consume\n");
+				OK = 0;
+				break;
+			};
+			amqp_destroy_envelope(&envelope);
+		}
+
+    	kz_amqp_connection_close(kzconn);
+    	kz_amqp_fire_connection_event("closed", kzconn->info->info.host);
+
     }
 }
