@@ -1,6 +1,4 @@
 /*
- * $Id$
- *
  * Copyright (C) 2001-2005 FhG Fokus
  * Copyright (C) 2005 Voice Sistem SRL
  * Copyright (C) 2010 Daniel-Constantin Mierla (asipto.com)
@@ -112,36 +110,6 @@ int pv_get_return_code(struct sip_msg *msg, pv_param_t *param,
 }
 */
 
-int pv_get_known_proto_string(int proto, str *sproto)
-{
-	switch(proto) {
-		case PROTO_UDP:
-			sproto->s = "udp";
-			sproto->len = 3;
-		return 0;
-		case PROTO_TCP:
-			sproto->s = "tcp";
-			sproto->len = 3;
-		return 0;
-		case PROTO_TLS:
-			sproto->s = "tls";
-			sproto->len = 3;
-		return 0;
-		case PROTO_SCTP:
-			sproto->s = "sctp";
-			sproto->len = 4;
-		return 0;
-		case PROTO_WS:
-			sproto->s = "ws";
-			sproto->len = 2;
-		return 0;
-		case PROTO_WSS:
-			sproto->s = "wss";
-			sproto->len = 3;
-		return 0;
-	}
-	return -1;
-}
 
 int pv_get_pid(struct sip_msg *msg, pv_param_t *param,
 		pv_value_t *res)
@@ -691,35 +659,44 @@ int pv_get_srcport(struct sip_msg *msg, pv_param_t *param,
 	return pv_get_uintval(msg, param, res, msg->rcv.src_port);
 }
 
-int pv_get_srcaddr_uri(struct sip_msg *msg, pv_param_t *param,
-		pv_value_t *res)
+int pv_get_srcaddr_uri_helper(struct sip_msg *msg, pv_param_t *param,
+		int tmode, pv_value_t *res)
 {
-	str sip;
-	str sproto;
+	str uri;
 	str sr;
 
 	if(msg==NULL)
 		return -1;
 
-	if(pv_get_known_proto_string(msg->rcv.proto, &sproto)<0)
+	if(get_src_uri(msg, tmode, &uri)<0)
 		return pv_get_null(msg, param, res);
 
-	sip.s = ip_addr2a(&msg->rcv.src_ip);
-	sip.len = strlen(sip.s);
-	if (sip.len + sproto.len + 32 >= pv_get_buffer_size())
+	if (uri.len + 1 >= pv_get_buffer_size())
 	{
 		LM_ERR("local buffer size exceeded\n");
 		return pv_get_null(msg, param, res);
 	}
 
 	sr.s = pv_get_buffer();
-	sr.len = snprintf(sr.s, pv_get_buffer_size(),
-			"sip:%.*s:%d;transport=%.*s", sip.len, sip.s,
-			msg->rcv.src_port, sproto.len, sproto.s);
+	strncpy(sr.s, uri.s, uri.len);
+	sr.len = uri.len;
+	sr.s[sr.len] = '\0';
 
 	return pv_get_strval(msg, param, res, &sr);
 }
 
+int pv_get_srcaddr_uri(struct sip_msg *msg, pv_param_t *param,
+		pv_value_t *res)
+{
+	return pv_get_srcaddr_uri_helper(msg, param, 0, res);
+}
+
+int pv_get_srcaddr_uri_full(struct sip_msg *msg, pv_param_t *param,
+		pv_value_t *res)
+{
+	return pv_get_srcaddr_uri_helper(msg, param, 1, res);
+}
+
 int pv_get_rcvip(struct sip_msg *msg, pv_param_t *param,
 		pv_value_t *res)
 {
@@ -1100,9 +1077,9 @@ int pv_get_proto(struct sip_msg *msg, pv_param_t *param,
 	if(msg==NULL)
 		return -1;
 
-	if(pv_get_known_proto_string(msg->rcv.proto, &s)<0)
+	if(get_valid_proto_string(msg->rcv.proto, 0, 0, &s)<0)
 	{
-		s.s = "NONE";
+		s.s = "none";
 		s.len = 4;
 	}
 
@@ -1804,6 +1781,9 @@ int pv_get_scriptvar(struct sip_msg *msg,  pv_param_t *param,
 	
 	sv= (script_var_t*)param->pvn.u.dname;
 
+	if((sv->v.flags&VAR_TYPE_NULL) && (sv->v.flags&VAR_VAL_NULL))
+			return pv_get_null(msg, param, res);
+
 	if(sv->v.flags&VAR_VAL_STR)
 	{
 		res->rs = sv->v.value.s;
@@ -1894,6 +1874,7 @@ int pv_get_tcpconn_id(struct sip_msg *msg, pv_param_t *param,
 		pv_value_t *res)
 {
 	struct tcp_connection *con;
+	int conid;
 
 	if (msg == NULL)
 		return -1;
@@ -1901,7 +1882,10 @@ int pv_get_tcpconn_id(struct sip_msg *msg, pv_param_t *param,
 	if ((con = tcpconn_get(msg->rcv.proto_reserved1, 0, 0, 0, 0)) == NULL)
 		return pv_get_null(msg, param, res);
 
-	return pv_get_sintval(msg, param, res, con->id);
+	conid = con->id;
+	tcpconn_put(con);
+
+	return pv_get_sintval(msg, param, res, conid);
 }
 
 
@@ -1984,8 +1968,13 @@ int pv_set_scriptvar(struct sip_msg* msg, pv_param_t *param,
 	}
 	if((val==NULL) || (val->flags&PV_VAL_NULL))
 	{
-		avp_val.n = 0;
-		set_var_value((script_var_t*)param->pvn.u.dname, &avp_val, 0);
+		if(((script_var_t*)param->pvn.u.dname)->v.flags&VAR_TYPE_NULL)
+		{
+			set_var_value((script_var_t*)param->pvn.u.dname, NULL, 0);
+		} else {
+			avp_val.n = 0;
+			set_var_value((script_var_t*)param->pvn.u.dname, &avp_val, 0);
+		}
 		return 0;
 	}
 	flags = 0;
@@ -2091,7 +2080,8 @@ int pv_set_ruri_user(struct sip_msg* msg, pv_param_t *param,
 		return -1;
 	}
 					
-	if((val==NULL) || (val->flags&PV_VAL_NULL))
+	if((val==NULL) || (val->flags&PV_VAL_NULL)
+			|| ((val->flags&PV_VAL_STR) && val->rs.len<=0))
 	{
 		memset(&act, 0, sizeof(act));
 		act.type = SET_USER_T;
@@ -2748,9 +2738,24 @@ int pv_parse_scriptvar_name(pv_spec_p sp, str *in)
 {
 	if(in==NULL || in->s==NULL || sp==NULL)
 		return -1;
-	
+
 	sp->pvp.pvn.type = PV_NAME_PVAR;
-	sp->pvp.pvn.u.dname = (void*)add_var(in);
+	sp->pvp.pvn.u.dname = (void*)add_var(in, VAR_TYPE_ZERO);
+	if(sp->pvp.pvn.u.dname==NULL)
+	{
+		LM_ERR("cannot register var [%.*s]\n", in->len, in->s);
+		return -1;
+	}
+	return 0;
+}
+
+int pv_parse_scriptvarnull_name(pv_spec_p sp, str *in)
+{
+	if(in==NULL || in->s==NULL || sp==NULL)
+		return -1;
+
+	sp->pvp.pvn.type = PV_NAME_PVAR;
+	sp->pvp.pvn.u.dname = (void*)add_var(in, VAR_TYPE_NULL);
 	if(sp->pvp.pvn.u.dname==NULL)
 	{
 		LM_ERR("cannot register var [%.*s]\n", in->len, in->s);
@@ -2903,8 +2908,7 @@ int pv_parse_flag_param(pv_spec_p sp, str *in)
 			LM_ERR("flag not declared: [%.*s]\n", in->len, in->s);
 			return -1;
 		}
-	}
-	if (check_flag(n) < 0)
+	} else if (check_flag(n) < 0)
 	{
 		LM_ERR("bad flag value: [%.*s]\n", in->len, in->s);
 		return -1;
