@@ -1,5 +1,4 @@
-/*$Id$
- *
+/*
  * Copyright (C) 2001-2003 FhG Fokus
  *
  * This file is part of Kamailio, a free SIP server.
@@ -19,35 +18,21 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  *
- * History:
- * -------
- *  2003-02-28  scratchpad compatibility abandoned (jiri)
- *  2003-01-29: - rewriting actions (replace, search_append) now begin
- *                at the second line -- previously, they could affect
- *                first line too, which resulted in wrong calculation of
- *                forwarded requests and an error consequently
- *              - replace_all introduced
- *  2003-01-28  scratchpad removed (jiri)
- *  2003-01-18  append_urihf introduced (jiri)
- *  2003-03-10  module export interface updated to the new format (andrei)
- *  2003-03-16  flags export parameter added (janakj)
- *  2003-03-19  replaced all mallocs/frees w/ pkg_malloc/pkg_free (andrei)
- *  2003-04-97  actions permitted to be used from failure/reply routes (jiri)
- *  2003-04-21  remove_hf and is_present_hf introduced (jiri)
- *  2003-08-19  subst added (support for sed like res:s/re/repl/flags) (andrei)
- *  2003-08-20  subst_uri added (like above for uris) (andrei)
- *  2003-09-11  updated to new build_lump_rpl() interface (bogdan)
- *  2003-11-11: build_lump_rpl() removed, add_lump_rpl() has flags (bogdan)
- *  2004-05-09: append_time introduced (jiri)
- *  2004-07-06  subst_user added (like subst_uri but only for user) (sobomax)
- *  2004-11-12  subst_user changes (old serdev mails) (andrei)
- *  2005-07-05  is_method("name") to check method using id (ramona)
- *  2006-03-17  applied patch from Marc Haisenko <haisenko@comdasys.com> 
- *              for adding has_body() function (bogdan)
- *  2008-07-14  Moved some function declarations to a separate file (Ardjan Zwartjes) 
- *
  */
 
+/*!
+ * \file
+ * \brief Module interface
+ * \ingroup textops
+ * Module: \ref textops
+ */
+
+/**
+ * @defgroup Textops Various text operatoins on messages
+ * @brief Kamailio textops module
+ */
+
+
 
 #include "../../action.h"
 #include "../../sr_module.h"
@@ -65,6 +50,7 @@
 #include "../../parser/parse_content.h"
 #include "../../parser/parse_param.h"
 #include "../../parser/sdp/sdp.h"
+#include "../../parser/sdp/sdp_helpr_funcs.h"
 #include "../../lib/kcore/parse_privacy.h"
 #include "../../msg_translator.h"
 #include "../../ut.h"
@@ -1455,8 +1441,8 @@ static int set_rpl_body_f(struct sip_msg* msg, char* p1, char* p2)
 	return 1;
 }
 
-static str* generate_boundary(str txt, str content_type,
-	str content_disposition, str delimiter, unsigned int initial)
+static str* generate_boundary(str *txt, str *content_type,
+	str *content_disposition, str *delimiter, unsigned int initial)
 {
 	unsigned int i = 0;
 	str cth = {"Content-Type: ", 14};
@@ -1464,32 +1450,37 @@ static str* generate_boundary(str txt, str content_type,
 	str* n;
 	unsigned int flag = 0;
 
-	if(txt.len==0||content_type.len==0||delimiter.len==0)
+	if(txt==NULL||txt->len==0
+		||content_type==NULL||content_type->len==0
+		||delimiter==NULL||delimiter->len==0)
 	{
 		LM_ERR("invalid parameters\n");
 		return NULL;
 	}
+	if (delimiter->s[0] == '-') {
+		LM_ERR("delimiter with initial '-'. Invalid parameter.\n");
+		return NULL;
+	}
 	n = pkg_malloc(sizeof(str));
 	if(n==NULL)
 	{
 		LM_ERR("out of pkg memory\n");
 		return NULL;
 	}
-	n->len = delimiter.len + 2 + CRLF_LEN;
+	n->len = delimiter->len + 2 + CRLF_LEN;
 	if(initial) n->len = 2*n->len;
-	if(strncmp("\r\n\r\n", txt.s+txt.len-4,4)!=0)
+	if(strncmp("\r\n\r\n", txt->s+txt->len-4,4)!=0)
 	{
 		n->len = n->len + CRLF_LEN;
 		flag = 1;
 		LM_DBG("adding final CRLF+CRLF\n");
 	}
-	n->len=n->len + cth.len + content_type.len + 2*CRLF_LEN;
-	if(content_disposition.len>0)
+	n->len=n->len + cth.len + content_type->len + 2*CRLF_LEN;
+	if(content_disposition->len>0)
 	{
-		n->len = n->len + cdh.len + content_disposition.len + CRLF_LEN;
+		n->len = n->len + cdh.len + content_disposition->len + CRLF_LEN;
 	}
-	n->len = n->len + txt.len;
-
+	n->len = n->len + txt->len;
 	n->s = pkg_malloc(sizeof(char)*(n->len));
 	if(n->s==0)
 	{
@@ -1497,31 +1488,32 @@ static str* generate_boundary(str txt, str content_type,
 		pkg_free(n);
 		return NULL;
 	}
+	memset(n->s, 0, sizeof(char)*n->len);
 	if(initial)
 	{
 		memcpy(n->s, "--", 2); i=2;
-		memcpy(n->s+i, delimiter.s, delimiter.len); i=i+delimiter.len;
+		memcpy(n->s+i, delimiter->s, delimiter->len); i=i+delimiter->len;
 		memcpy(n->s+i, CRLF, CRLF_LEN); i=i+CRLF_LEN;
 	}
 
 	memcpy(n->s+i, cth.s, cth.len); i=i+cth.len;
-	memcpy(n->s+i, content_type.s, content_type.len); i=i+content_type.len;
+	memcpy(n->s+i, content_type->s, content_type->len); i=i+content_type->len;
 	memcpy(n->s+i, CRLF, CRLF_LEN); i=i+CRLF_LEN;
 
-	if(content_disposition.len>0)
+	if(content_disposition->len>0)
 	{
 		memcpy(n->s+i, cdh.s, cdh.len); i=i+cdh.len;
-		memcpy(n->s+i, content_disposition.s, content_disposition.len);
-		i=i+content_disposition.len;
+		memcpy(n->s+i, content_disposition->s, content_disposition->len);
+		i=i+content_disposition->len;
 		memcpy(n->s+i, CRLF, CRLF_LEN); i=i+CRLF_LEN;
 	}
 	memcpy(n->s+i, CRLF, CRLF_LEN); i=i+CRLF_LEN;
 
-	memcpy(n->s+i, txt.s, txt.len); i=i+txt.len;
+	memcpy(n->s+i, txt->s, txt->len); i=i+txt->len;
 	if(flag) { memcpy(n->s+i, CRLF, CRLF_LEN); i=i+CRLF_LEN; }
 
 	memcpy(n->s+i, "--", 2); i=i+2;
-	memcpy(n->s+i, delimiter.s, delimiter.len); i=i+delimiter.len;
+	memcpy(n->s+i, delimiter->s, delimiter->len); i=i+delimiter->len;
 	memcpy(n->s+i, CRLF, CRLF_LEN); i=i+CRLF_LEN;
 
 	if(i!=n->len)
@@ -1668,7 +1660,7 @@ int set_multibody_helper(struct sip_msg* msg, char* p1, char* p2, char* p3)
 	}
 
 	/* get initial boundary */
-	nbb = generate_boundary(nb, oc, cd, delimiter, 1);
+	nbb = generate_boundary(&nb, &oc, &cd, &delimiter, 1);
 	if(nbb==NULL)
 	{
 		LM_ERR("couldn't create initial boundary\n");
@@ -1857,19 +1849,17 @@ int append_multibody_helper(struct sip_msg* msg, char* p1, char* p2, char* p3)
 		LM_ERR("WTF\n");
 		return -1;
 	}
-	/* get boundary */
-	if(get_boundary(msg, &delimiter)!=0) {
+	/* get delimiter no initial -- */
+	if(get_mixed_part_delimiter(&msg->content_type->body, &delimiter) < 0) {
 		LM_ERR("Cannot get boundary. Is body multipart?\n");
 		return -1;
 	}
-	nbb = generate_boundary(txt, nc, cd, delimiter, 0);
+	nbb = generate_boundary(&txt, &nc, &cd, &delimiter, 0);
 	if(nbb==NULL)
 	{
 		LM_ERR("couldn't create initial boundary\n");
-		pkg_free(delimiter.s);
 		return -1;
 	}
-	pkg_free(delimiter.s);
 	if(insert_new_lump_after(l, nbb->s, nbb->len, 0)==0){
 		LM_ERR("could not insert new lump\n");
 		pkg_free(nbb->s); pkg_free(nbb);
@@ -2121,6 +2111,11 @@ int add_hf_helper(struct sip_msg* msg, str *str1, str *str2,
 	len=s0.len;
 	if (str2) len+= str2->len + REQ_LINE(msg).uri.len;
 
+	if(len==0) {
+		LM_INFO("nothing to add\n");
+		return -1;
+	}
+
 	s  = (char*)pkg_malloc(len);
 	if (!s) {
 		LM_ERR("no pkg memory left\n");
@@ -2774,7 +2769,7 @@ static int search_hf_f(struct sip_msg* msg, char* str_hf, char* re, char *flags)
 					return 1;
 			} else {
 				if(flags!=NULL && *flags=='f')
-					return 1;
+					return -1;
 			}
 		} else {
 			hfl = hf;
@@ -2884,6 +2879,9 @@ static int subst_hf_f(struct sip_msg *msg, char *str_hf, char *subst, char *flag
 		} else {
 			hfl = hf;
 		}
+		/* if flags set for first header, then all done */
+		if(flags!=NULL && *flags=='f')
+			return ret;
 	}
 	if(hfl!=NULL)
 	{
