@@ -1,22 +1,14 @@
 /*
- * $Id$
- *
- *
  * Copyright (C) 2001-2003 FhG Fokus
  *
- * This file is part of ser, a free SIP server.
+ * This file is part of Kamailio, a free SIP server.
  *
- * ser is free software; you can redistribute it and/or modify
+ * Kamailio is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version
  *
- * For a license to use the ser software under conditions
- * other than those described here, or to purchase support for this
- * software, please contact iptel.org by e-mail at the following addresses:
- *    info@iptel.org
- *
- * ser is distributed in the hope that it will be useful,
+ * Kamailio is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -25,66 +17,6 @@
  * along with this program; if not, write to the Free Software 
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
-/*
- * History:
- * -------
- *  2003-02-13  proto support added (andrei)
- *  2003-02-24  s/T_NULL/T_NULL_CELL/ to avoid redefinition conflict w/
- *              nameser_compat.h (andrei)
- *  2003-03-01  kr set through a function now (jiri)
- *  2003-03-06  callbacks renamed; "blind UAC" introduced, which makes
- *              transaction behave as if it was forwarded even if it was
- *              not -- good for local UAS, like VM (jiri)
- *  2003-03-19  replaced all the mallocs/frees w/ pkg_malloc/pkg_free (andrei)
- *  2003-03-30  we now watch downstream delivery and if it fails, send an
- *              error message upstream (jiri)
- *  2003-04-14  use protocol from uri (jiri)
- *  2003-12-04  global TM callbacks switched to per transaction callbacks
- *              (bogdan)
- *  2004-02-13: t->is_invite and t->local replaced with flags (bogdan)
- *  2005-08-04  msg->parsed_uri and parsed_uri_ok are no saved & restored
- *               before & after handling the branches (andrei)
- *  2005-12-11  onsend_route support added for forwarding (andrei)
- *  2006-01-27  t_forward_no_ack will return error if a forward on an 
- *              already canceled transaction is attempted (andrei)
- *  2006-02-07  named routes support (andrei)
- *  2006-04-18  add_uac simplified + switched to struct dest_info (andrei)
- *  2006-04-20  pint_uac_request uses now struct dest_info (andrei)
- *  2006-08-11  dns failover support (andrei)
- *              t_forward_non_ack won't start retransmission on send errors
- *               anymore (WARNING: callers should release/kill the transaction
- *               if error is returned) (andrei)
- *  2006-09-15  e2e_cancel uses t_reply_unsafe when called from the 
- *               failure_route and replying to a cancel (andrei)
- *  2006-10-10  e2e_cancel update for the new/modified 
- *               which_cancel()/should_cancel() (andrei)
- *  2006-10-11  don't fork a new branch if the transaction or branch was
- *               canceled, or a 6xx was received
- *              stop retr. timers fix on cancel for non-invites     (andrei)
- *  2006-11-20  new_uri is no longer saved/restore across add_uac calls, since
- *              print_uac_request is now uri safe (andrei)
- *  2007-03-15  TMCB_ONSEND hooks added (andrei)
- *  2007-05-02  added t_forward_cancel(unmatched_cancel) (andrei)
- *  2007-05-24  added TMCB_E2ECANCEL_IN hook support (andrei)
- *  2007-05-28: e2e_cancel_branch() constructs the CANCEL from the
- *              outgoing INVITE instead of applying the lumps to the
- *              incomming one. (it can be disabled with reparse_invite=0) (Miklos)
- *              t_relay_cancel() introduced -- can be used to relay CANCELs
- *              at the beginning of the script. (Miklos)
- * 2007-06-04  running transaction are canceled hop by hop (andrei)
- * 2007-08-37  In case of DNS failover the new SIP message is constructed
- *              from the message buffer of the failed branch instead of
- *              applying the lumps again, because the per-branch lumps are no
- *              t saved, thus, are not available. Set reparse_on_dns_failover
- *              to 0 to revert the change. (Miklos)
- * 2008-06-04  T_CANCELED is now set each time a CANCEL is received (andrei)
- * 2009-06-01  Pre- and post-script callbacks of branch route are 
- *             executed (Miklos)
- * 2009-10-26  support for changing dst_uri in branch routes,
- *             s/print_uac_request/prepare_new_uac/ (andrei)
- * 2009-10-28  support for changing r-uris and path in branch routes; more of
- *             add_uac() functionality moved into prepare_new_uac()  (andrei)
- */
 
 #include "defs.h"
 
@@ -112,6 +44,8 @@
 #include "t_cancel.h"
 #include "t_lookup.h"
 #include "t_fwd.h"
+#include "t_reply.h"
+#include "h_table.h"
 #include "../../fix_lumps.h"
 #include "config.h"
 #ifdef USE_DNS_FAILOVER
@@ -600,6 +534,15 @@ static int prepare_new_uac( struct cell *t, struct sip_msg *i_req,
 		t->uac[branch].location_ua.s[i_req->location_ua.len]=0;
 		memcpy( t->uac[branch].location_ua.s, i_req->location_ua.s, i_req->location_ua.len);
 	}
+
+#ifdef TM_UAC_FLAGS
+	len = count_applied_lumps(i_req->add_rm, HDR_RECORDROUTE_T);
+	if(len==1)
+		t->uac[branch].flags = TM_UAC_FLAG_RR;
+	else if(len==2)
+		t->uac[branch].flags = TM_UAC_FLAG_RR|TM_UAC_FLAG_R2;
+#endif
+
 	ret=0;
 
 error01:
@@ -746,7 +689,7 @@ int add_blind_uac( /*struct cell *t*/ )
 	}
 
 	branch=t->nr_of_outgoings;	
-	if (branch==MAX_BRANCHES) {
+	if (branch==sr_dst_max_branches) {
 		LOG(L_ERR, "ERROR: add_blind_uac: "
 			"maximum number of branches exceeded\n");
 		return -1;
@@ -806,12 +749,9 @@ int add_uac( struct cell *t, struct sip_msg *request, str *uri,
 
 	int ret;
 	unsigned short branch;
-#ifdef TM_UAC_FLAGS
-	unsigned int len;
-#endif /* TM_UAC_FLAGS */
 
 	branch=t->nr_of_outgoings;
-	if (branch==MAX_BRANCHES) {
+	if (branch==sr_dst_max_branches) {
 		LOG(L_ERR, "ERROR: add_uac: maximum number of branches exceeded\n");
 		ret=ser_error=E_TOO_MANY_BRANCHES;
 		goto error;
@@ -851,13 +791,6 @@ int add_uac( struct cell *t, struct sip_msg *request, str *uri,
 		ser_error=ret;
 		goto error01;
 	}
-#ifdef TM_UAC_FLAGS
-	len = count_applied_lumps(request->add_rm, HDR_RECORDROUTE_T);
-	if(len==1)
-		t->uac[branch].flags = TM_UAC_FLAG_RR;
-	else if(len==2)
-		t->uac[branch].flags = TM_UAC_FLAG_RR|TM_UAC_FLAG_R2;
-#endif
 	getbflagsval(0, &t->uac[branch].branch_flags);
 	membar_write(); /* to allow lockless ops (e.g. prepare_to_cancel()) we want
 					   to be sure everything above is fully written before
@@ -898,7 +831,7 @@ static int add_uac_from_buf( struct cell *t, struct sip_msg *request,
 	unsigned int len;
 
 	branch=t->nr_of_outgoings;
-	if (branch==MAX_BRANCHES) {
+	if (branch==sr_dst_max_branches) {
 		LOG(L_ERR, "ERROR: add_uac_from_buf: maximum number of branches"
 					" exceeded\n");
 		ret=ser_error=E_TOO_MANY_BRANCHES;
@@ -1060,7 +993,7 @@ int add_uac_dns_fallback(struct cell *t, struct sip_msg* msg,
 					return ret;
 				}
 			}
-			if (t->nr_of_outgoings >= MAX_BRANCHES){
+			if (t->nr_of_outgoings >= sr_dst_max_branches){
 				LOG(L_ERR, "ERROR: add_uac_dns_fallback: maximum number of "
 							"branches exceeded\n");
 				if (lock_replies)
@@ -1605,7 +1538,7 @@ int t_send_branch( struct cell *t, int branch, struct sip_msg* p_msg ,
 		}
 #endif
 		uac->icode = 908; /* internal code set to delivery failure */
-		LOG(L_ERR, "ERROR: t_send_branch: sending request on branch %d "
+		LOG(L_WARN, "ERROR: t_send_branch: sending request on branch %d "
 				"failed\n", branch);
 		if (proxy) { proxy->errors++; proxy->ok=0; }
 		if(tm_failure_exec_mode==1) {
