@@ -1,21 +1,14 @@
 /*
- * $Id$
- *
  * Copyright (C) 2001-2003 FhG Fokus
  *
- * This file is part of ser, a free SIP server.
+ * This file is part of Kamailio, a free SIP server.
  *
- * ser is free software; you can redistribute it and/or modify
+ * Kamailio is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version
  *
- * For a license to use the ser software under conditions
- * other than those described here, or to purchase support for this
- * software, please contact iptel.org by e-mail at the following addresses:
- *    info@iptel.org
- *
- * ser is distributed in the hope that it will be useful,
+ * Kamailio is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -24,21 +17,6 @@
  * along with this program; if not, write to the Free Software 
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
-/*
- * History:
- * --------
- *  2003-03-19  replaced all the mallocs/frees w/ pkg_malloc/pkg_free (andrei)
- *  2003-12-04  global callbacks moved into transaction callbacks;
- *              multiple events per callback added; single list per
- *              transaction for all its callbacks (bogdan)
- *  2004-08-23  user avp(attribute value pair) added -> making avp list
- *              available in callbacks (bogdan)
- * 2007-03-08  membar_write() used in insert_tmcb(...) (andrei)
- * 2007-03-14  added *_SENT callbacks (andrei)
- * 2007-03-23  added local_req_in callbacks support (andrei)
- * 2007-05-17  insert_tmcb is now safe: it loops arround a mb_atomic_cmpxchg
- *              for a safe lockless list insert (andrei)
- */
 
 #include "defs.h"
 
@@ -355,6 +333,9 @@ static void run_reqin_callbacks_internal(struct tmcb_head_list* hl,
 	struct tm_callback    *cbp;
 	avp_list_t* backup_from, *backup_to, *backup_dom_from, *backup_dom_to,
 				*backup_uri_from, *backup_uri_to;
+#ifdef WITH_XAVP
+	sr_xavp_t **backup_xavps;
+#endif
 
 	if (hl==0 || hl->first==0) return;
 	backup_uri_from = set_avp_list(AVP_CLASS_URI | AVP_TRACK_FROM,
@@ -369,6 +350,9 @@ static void run_reqin_callbacks_internal(struct tmcb_head_list* hl,
 			&trans->domain_avps_from);
 	backup_dom_to = set_avp_list(AVP_CLASS_DOMAIN | AVP_TRACK_TO, 
 			&trans->domain_avps_to);
+#ifdef WITH_XAVP
+	backup_xavps = xavp_set_list(&trans->xavps_list);
+#endif
 	for (cbp=(struct tm_callback*)hl->first; cbp; cbp=cbp->next)  {
 		DBG("DBG: trans=%p, callback type %d, id %d entered\n",
 			trans, cbp->types, cbp->id );
@@ -381,6 +365,9 @@ static void run_reqin_callbacks_internal(struct tmcb_head_list* hl,
 	set_avp_list(AVP_CLASS_DOMAIN | AVP_TRACK_FROM, backup_dom_from );
 	set_avp_list(AVP_CLASS_USER | AVP_TRACK_TO, backup_to );
 	set_avp_list(AVP_CLASS_USER | AVP_TRACK_FROM, backup_from );
+#ifdef WITH_XAVP
+	xavp_set_list(backup_xavps);
+#endif
 }
 
 
