@@ -1,6 +1,4 @@
 /**
- * $Id$
- *
  * Copyright (C) 2010 Daniel-Constantin Mierla (asipto.com)
  *
  * This file is part of kamailio, a free SIP server.
@@ -118,6 +116,7 @@ str auth_password_column = str_init("auth_password");
 str auth_proxy_column = str_init("auth_proxy");
 str expires_column = str_init("expires");
 
+
 #if 0
 INSERT INTO version (table_name, table_version) values ('uacreg','1');
 CREATE TABLE uacreg (
@@ -142,9 +141,22 @@ extern pv_spec_t auth_username_spec;
 extern pv_spec_t auth_realm_spec;
 extern pv_spec_t auth_password_spec;
 
+counter_handle_t regtotal;         /* Total number of registrations in memory */
+counter_handle_t regactive;        /* Active registrations - 200 OK */
+counter_handle_t regdisabled;      /* Disabled registrations */
+
+/* Init counters */
+static void uac_reg_counter_init()
+{
+	LM_DBG("*** Initializing UAC reg counters\n");
+        counter_register(&regtotal, "uac", "regtotal", 0, 0, 0, "Total number of registration accounts in memory", 0);
+        counter_register(&regactive, "uac", "regactive", 0, 0, 0, "Number of successfully registred accounts (200 OK)", 0);
+        counter_register(&regdisabled, "uac", "regdisabled", 0, 0, 0, "Counter of failed registrations (not 200 OK)", 0);
+}
+
 
 /**
- *
+ * Init the in-memory registration database in hash table
  */
 int uac_reg_init_ht(unsigned int sz)
 {
@@ -216,7 +228,7 @@ int uac_reg_init_ht(unsigned int sz)
 	{
 		if(lock_init(&_reg_htable->entries[i].lock)==0)
 		{
-			LM_ERR("cannot initalize lock[%d] n", i);
+			LM_ERR("cannot initialize lock[%d] n", i);
 			i--;
 			while(i>=0)
 			{
@@ -233,6 +245,9 @@ int uac_reg_init_ht(unsigned int sz)
 		}
 	}
 
+	/* Initialize uac reg counters */
+	uac_reg_counter_init();
+
 	return 0;
 }
 
@@ -345,6 +360,10 @@ int uac_reg_reset_ht_gc(void)
 		_reg_htable_gc->entries[i].byuser = NULL;
 		_reg_htable_gc->entries[i].usize = 0;
 	}
+	/* Reset all counters */
+	counter_reset(regtotal);
+	counter_reset(regactive);
+	counter_reset(regdisabled);
 	return 0;
 }
 
@@ -366,7 +385,7 @@ int uac_reg_ht_shift(void)
 	lock_get(_reg_htable_gc_lock);
 	if(_reg_htable_gc->stime > tn-UAC_REG_GC_INTERVAL) {
 		lock_release(_reg_htable_gc_lock);
-		LM_ERR("shifting the memory table is not possible in less than %d\n", UAC_REG_GC_INTERVAL);
+		LM_ERR("shifting the memory table is not possible in less than %d secs\n", UAC_REG_GC_INTERVAL);
 		return -1;
 	}
 	uac_reg_reset_ht_gc();
@@ -518,6 +537,7 @@ int reg_ht_add(reg_uac_t *reg)
 
 	reg_ht_add_byuser(nr);
 	reg_ht_add_byuuid(nr);
+	counter_inc(regtotal);
 
 	return 0;
 }
@@ -878,14 +898,17 @@ void uac_reg_tm_callback( struct cell *t, int type, struct tmcb_params *ps)
 	}
 
 error:
-	if(reg_retry_interval)
+	if(reg_retry_interval) {
 		ri->timer_expires = time(NULL) + reg_retry_interval;
-	else
+	} else {
 		ri->flags |= UAC_REG_DISABLED;
+		counter_inc(regdisabled);
+	}
 done:
 	if(ri)
 		ri->flags &= ~(UAC_REG_ONGOING|UAC_REG_AUTHSENT);
 	shm_free(uuid);
+	counter_inc(regactive);
 }
 
 int uac_reg_update(reg_uac_t *reg, time_t tn)
@@ -911,15 +934,16 @@ int uac_reg_update(reg_uac_t *reg, time_t tn)
 		return 4;
 	if(reg->timer_expires > tn + reg_timer_interval + 3)
 		return 3;
-	reg->timer_expires = tn;
-	reg->flags |= UAC_REG_ONGOING;
-	reg->flags &= ~UAC_REG_ONLINE;
 	uuid = (char*)shm_malloc(reg->l_uuid.len+1);
 	if(uuid==NULL)
 	{
 		LM_ERR("no more shm\n");
 		return -1;
 	}
+	reg->timer_expires = tn;
+	reg->flags |= UAC_REG_ONGOING;
+	reg->flags &= ~UAC_REG_ONLINE;
+	counter_add(regactive, -1);		/* Take it out of the active pool while re-registering */
 	memcpy(uuid, reg->l_uuid.s, reg->l_uuid.len);
 	uuid[reg->l_uuid.len] = '\0';
 
@@ -961,8 +985,10 @@ int uac_reg_update(reg_uac_t *reg, time_t tn)
 		shm_free(uuid);
 		if (reg_retry_interval)
 			reg->timer_expires = (tn ? tn : time(NULL)) + reg_retry_interval;
-		else
+		else {
 			reg->flags |= UAC_REG_DISABLED;
+			counter_inc(regdisabled);
+		}
 		reg->flags &= ~UAC_REG_ONGOING;
 		return -1;
 	}
@@ -1009,9 +1035,9 @@ void uac_reg_timer(unsigned int ticks)
 				(char*)(RES_ROWS(db_res)[i].values[pos].val.string_val); \
 			reg.attr.len = strlen(reg.attr.s); \
 			if(reg.attr.len == 0) { \
-				LM_ERR("empty value not allowed for column[%d]=%.*s\n", \
+				LM_ERR("empty value not allowed for column[%d]='%.*s' - ignoring record\n", \
 						pos, db_cols[pos]->len, db_cols[pos]->s); \
-				goto error; \
+				continue; \
 			} \
 		} \
 	} while(0);
@@ -1251,6 +1277,8 @@ int uac_reg_db_refresh(str *pl_uuid)
 	reg_db_set_attr(auth_password, 7);
 	reg_db_set_attr(auth_proxy, 8);
 	reg.expires = (unsigned int)RES_ROWS(db_res)[i].values[9].val.int_val;
+	reg.h_uuid = reg_compute_hash(&reg.l_uuid);
+	reg.h_user = reg_compute_hash(&reg.l_username);
 
 	lock_get(_reg_htable_gc_lock);
 	if(reg_ht_get_byuuid(pl_uuid)!=NULL)
@@ -1572,6 +1600,7 @@ static void rpc_uac_reg_info(rpc_t* rpc, void* ctx)
 					rpc->fault(ctx, 500, "Internal error adding item");
 					return;
 				}
+				lock_release(&_reg_htable->entries[i].lock);
 				return;
 			}
 			reg = reg->next;
@@ -1653,7 +1682,8 @@ static const char* rpc_uac_reg_enable_doc[2] = {
 
 static void rpc_uac_reg_enable(rpc_t* rpc, void* ctx)
 {
-	rpc_uac_reg_update_flag(rpc, ctx, 1, UAC_REG_DISABLED);
+	rpc_uac_reg_update_flag(rpc, ctx, 0, UAC_REG_DISABLED);
+	counter_add(regdisabled, -1);
 }
 
 static const char* rpc_uac_reg_disable_doc[2] = {
@@ -1663,7 +1693,8 @@ static const char* rpc_uac_reg_disable_doc[2] = {
 
 static void rpc_uac_reg_disable(rpc_t* rpc, void* ctx)
 {
-	rpc_uac_reg_update_flag(rpc, ctx, 0, UAC_REG_DISABLED);
+	rpc_uac_reg_update_flag(rpc, ctx, 1, UAC_REG_DISABLED);
+	counter_inc(regdisabled);
 }
 
 static const char* rpc_uac_reg_reload_doc[2] = {
