@@ -16,24 +16,12 @@
 
 /**
  * @file 
- * @brief SIP-router core :: rvalue expressions
+ * @brief Kamailio core :: rvalue expressions
  * @ingroup core
  * Module: \ref core
  */
 
-/* 
- * History:
- * --------
- *  2008-12-01  initial version (andrei)
- *  2009-04-24  added support for defined, strempty, strlen (andrei)
- *  2009-04-28  int and str automatic conversions: (int)undef=0,
- *               (str)undef="", (int)""=0, (int)"123"=123, (int)"abc"=0
- *              handle undef == expr, in function of the UNDEF_EQ_* defines.
- *              (andrei)
- *  2009-05-05  casts operator for int & string (andrei)
- */
-
-/* special defines:
+/** special defines:
  *
  *  UNDEF_EQ_* - how to behave when undef is on the right side of a generic
  *               compare operator
@@ -527,6 +515,7 @@ enum rval_type rve_guess_type( struct rval_expr* rve)
 		case RVE_STRLEN_OP:
 		case RVE_STREMPTY_OP:
 		case RVE_DEFINED_OP:
+		case RVE_NOTDEFINED_OP:
 		case RVE_INT_OP:
 			return RV_INT;
 		case RVE_PLUS_OP:
@@ -572,6 +561,7 @@ int rve_is_constant(struct rval_expr* rve)
 		case RVE_STRLEN_OP:
 		case RVE_STREMPTY_OP:
 		case RVE_DEFINED_OP:
+		case RVE_NOTDEFINED_OP:
 		case RVE_INT_OP:
 		case RVE_STR_OP:
 			return rve_is_constant(rve->left.rve);
@@ -636,6 +626,7 @@ static int rve_op_unary(enum rval_expr_op op)
 		case RVE_STRLEN_OP:
 		case RVE_STREMPTY_OP:
 		case RVE_DEFINED_OP:
+		case RVE_NOTDEFINED_OP:
 		case RVE_INT_OP:
 		case RVE_STR_OP:
 			return 1;
@@ -839,6 +830,7 @@ int rve_check_type(enum rval_type* type, struct rval_expr* rve,
 		case RVE_STRLEN_OP:
 		case RVE_STREMPTY_OP:
 		case RVE_DEFINED_OP:
+		case RVE_NOTDEFINED_OP:
 			*type=RV_INT;
 			if (rve_check_type(&type1, rve->left.rve, bad_rve, bad_t, exp_t)){
 				if (type1==RV_INT){
@@ -2112,6 +2104,10 @@ int rval_expr_eval_int( struct run_act_ctx* h, struct sip_msg* msg,
 		case RVE_DEFINED_OP:
 			ret=int_rve_defined(h, msg, res, rve->left.rve);
 			break;
+		case RVE_NOTDEFINED_OP:
+			ret=int_rve_defined(h, msg, res, rve->left.rve);
+			*res = !(*res);
+			break;
 		case RVE_STREQ_OP:
 		case RVE_STRDIFF_OP:
 		case RVE_MATCH_OP:
@@ -2233,6 +2229,7 @@ int rval_expr_eval_rvint(			   struct run_act_ctx* h,
 		case RVE_STRLEN_OP:
 		case RVE_STREMPTY_OP:
 		case RVE_DEFINED_OP:
+		case RVE_NOTDEFINED_OP:
 		case RVE_INT_OP:
 			/* operator forces integer type */
 			ret=rval_expr_eval_int(h, msg, res_i, rve);
@@ -2360,6 +2357,7 @@ struct rvalue* rval_expr_eval(struct run_act_ctx* h, struct sip_msg* msg,
 		case RVE_STRLEN_OP:
 		case RVE_STREMPTY_OP:
 		case RVE_DEFINED_OP:
+		case RVE_NOTDEFINED_OP:
 		case RVE_INT_OP:
 			/* operator forces integer type */
 			r=rval_expr_eval_int(h, msg, &i, rve);
@@ -2544,6 +2542,7 @@ struct rval_expr* mk_rval_expr_v(enum rval_type rv_type, void* val,
 			s=(str*)val;
 			v.s.s=pkg_malloc(s->len+1 /*0*/);
 			if (v.s.s==0){
+				pkg_free(rve);
 				ERR("memory allocation failure\n");
 				return 0;
 			}
@@ -2601,6 +2600,7 @@ struct rval_expr* mk_rval_expr1(enum rval_expr_op op, struct rval_expr* rve1,
 		case RVE_STRLEN_OP:
 		case RVE_STREMPTY_OP:
 		case RVE_DEFINED_OP:
+		case RVE_NOTDEFINED_OP:
 		case RVE_INT_OP:
 		case RVE_STR_OP:
 			break;
@@ -2692,6 +2692,7 @@ static int rve_op_is_assoc(enum rval_expr_op op)
 		case RVE_STRLEN_OP:
 		case RVE_STREMPTY_OP:
 		case RVE_DEFINED_OP:
+		case RVE_NOTDEFINED_OP:
 		case RVE_INT_OP:
 		case RVE_STR_OP:
 			/* one operand expression => cannot be assoc. */
@@ -2747,6 +2748,7 @@ static int rve_op_is_commutative(enum rval_expr_op op)
 		case RVE_STRLEN_OP:
 		case RVE_STREMPTY_OP:
 		case RVE_DEFINED_OP:
+		case RVE_NOTDEFINED_OP:
 		case RVE_INT_OP:
 		case RVE_STR_OP:
 			/* one operand expression => cannot be commut. */
@@ -2815,7 +2817,7 @@ static int rve_can_optimize_int(struct rval_expr* rve)
 		if (rve->right.rve->left.rval.type!=RV_INT)
 			return 0;
 	}
-	DBG("rve_can_optimize_int: left %d, right %d\n", 
+	LM_DBG("left %d, right %d\n", 
 			rve->left.rve->op, rve->right.rve?rve->right.rve->op:0);
 	return 1;
 }
@@ -2833,7 +2835,7 @@ static int rve_can_optimize_str(struct rval_expr* rve)
 		return 0;
 	if (rve->op == RVE_RVAL_OP)
 		return 0;
-	DBG("rve_can_optimize_str: left %d, right %d\n", 
+	LM_DBG("left %d, right %d\n", 
 			rve->left.rve->op, rve->right.rve?rve->right.rve->op:0);
 	if (rve->left.rve->op != RVE_RVAL_OP)
 		return 0;
@@ -2854,15 +2856,15 @@ static int rve_can_optimize_str(struct rval_expr* rve)
 
 static int fix_rval(struct rvalue* rv)
 {
-	DBG("RV fixing type %d\n", rv->type);
+	LM_DBG("RV fixing type %d\n", rv->type);
 	switch(rv->type){
 		case RV_INT:
 			/*nothing to do*/
-			DBG("RV is int: %d\n", (int)rv->v.l);
+			LM_DBG("RV is int: %d\n", (int)rv->v.l);
 			return 0;
 		case RV_STR:
 			/*nothing to do*/
-			DBG("RV is str: \"%s\"\n", rv->v.s.s);
+			LM_DBG("RV is str: \"%s\"\n", rv->v.s.s);
 			return 0;
 		case RV_BEXPR:
 			return fix_expr(rv->v.bexpr);
@@ -3261,13 +3263,13 @@ static int rve_opt_01(struct rval_expr* rve, enum rval_type rve_type)
 			if (right){
 				if (rve->op==RVE_RVAL_OP){
 					if (rve->left.rval.type==RV_INT)
-						DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
+						LM_DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
 								" op%d($v, %d) -> %d\n", 
 								rve->fpos.s_line, rve->fpos.s_col,
 								rve->fpos.e_line, rve->fpos.e_col,
 								op, i, (int)rve->left.rval.v.l);
 					else
-						DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
+						LM_DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
 								" op%d($v, %d) -> $v (rval)\n",
 								rve->fpos.s_line, rve->fpos.s_col,
 								rve->fpos.e_line, rve->fpos.e_col,
@@ -3275,19 +3277,19 @@ static int rve_opt_01(struct rval_expr* rve, enum rval_type rve_type)
 				}else if (rve->op==RVE_INT_OP){
 					if (rve->left.rve->op==RVE_RVAL_OP &&
 							rve->left.rve->left.rval.type==RV_INT)
-						DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
+						LM_DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
 								" op%d($v, %d) -> (int)%d\n", 
 								rve->fpos.s_line, rve->fpos.s_col,
 								rve->fpos.e_line, rve->fpos.e_col,
 								op, i, (int)rve->left.rve->left.rval.v.l);
 					else
-						DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
+						LM_DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
 								" op%d($v, %d) -> (int)$v\n",
 								rve->fpos.s_line, rve->fpos.s_col,
 								rve->fpos.e_line, rve->fpos.e_col,
 								op, i);
 				}else{
-					DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
+					LM_DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
 							" op%d($v, %d) -> $v\n",
 							rve->fpos.s_line, rve->fpos.s_col,
 							rve->fpos.e_line, rve->fpos.e_col,
@@ -3296,13 +3298,13 @@ static int rve_opt_01(struct rval_expr* rve, enum rval_type rve_type)
 			}else{
 				if (rve->op==RVE_RVAL_OP){
 					if (rve->left.rval.type==RV_INT)
-						DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
+						LM_DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
 								" op%d(%d, $v) -> %d\n", 
 								rve->fpos.s_line, rve->fpos.s_col,
 								rve->fpos.e_line, rve->fpos.e_col,
 								op, i, (int)rve->left.rval.v.l);
 					else
-						DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
+						LM_DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
 								" op%d(%d, $v) -> $v (rval)\n",
 								rve->fpos.s_line, rve->fpos.s_col,
 								rve->fpos.e_line, rve->fpos.e_col,
@@ -3310,19 +3312,19 @@ static int rve_opt_01(struct rval_expr* rve, enum rval_type rve_type)
 				}else if (rve->op==RVE_INT_OP){
 					if (rve->left.rve->op==RVE_RVAL_OP &&
 							rve->left.rve->left.rval.type==RV_INT)
-						DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
+						LM_DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
 								" op%d(%d, $v) -> (int)%d\n", 
 								rve->fpos.s_line, rve->fpos.s_col,
 								rve->fpos.e_line, rve->fpos.e_col,
 								op, i, (int)rve->left.rve->left.rval.v.l);
 					else
-						DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
+						LM_DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
 								" op%d(%d, $v) -> (int)$v\n",
 								rve->fpos.s_line, rve->fpos.s_col,
 								rve->fpos.e_line, rve->fpos.e_col,
 								op, i);
 				}else{
-					DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
+					LM_DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
 							" op%d(%d, $v) -> $v\n",
 							rve->fpos.s_line, rve->fpos.s_col,
 							rve->fpos.e_line, rve->fpos.e_col,
@@ -3353,7 +3355,7 @@ static int rve_opt_01(struct rval_expr* rve, enum rval_type rve_type)
 					rve->right.rve=0;
 					ret=1;
 					if (dbg)
-						DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
+						LM_DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
 								" op%d($v, \"\") -> strempty($v)\n", 
 								rve->fpos.s_line, rve->fpos.s_col,
 								rve->fpos.e_line, rve->fpos.e_col,
@@ -3374,31 +3376,31 @@ static int rve_opt_01(struct rval_expr* rve, enum rval_type rve_type)
 			if (right){
 				if (rve->op==RVE_RVAL_OP){
 					if (rve->left.rval.type==RV_STR)
-						DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
+						LM_DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
 								" op%d($v, <string>) -> \"%s\"\n", 
 								rve->fpos.s_line, rve->fpos.s_col,
 								rve->fpos.e_line, rve->fpos.e_col,
 								op, rve->left.rval.v.s.s);
 					else
-						DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
+						LM_DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
 								" op%d($v, <string>) -> $v (rval)\n",
 								rve->fpos.s_line, rve->fpos.s_col,
 								rve->fpos.e_line, rve->fpos.e_col, op);
 				}else if (rve->op==RVE_STR_OP){
 					if (rve->left.rve->op==RVE_RVAL_OP &&
 							rve->left.rve->left.rval.type==RV_STR)
-						DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
+						LM_DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
 								" op%d($v, <string>) -> (str)\"%s\"\n", 
 								rve->fpos.s_line, rve->fpos.s_col,
 								rve->fpos.e_line, rve->fpos.e_col,
 								op, rve->left.rve->left.rval.v.s.s);
 					else
-						DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
+						LM_DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
 								" op%d($v, <string>) -> (str)$v\n",
 								rve->fpos.s_line, rve->fpos.s_col,
 								rve->fpos.e_line, rve->fpos.e_col, op);
 				}else{
-					DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
+					LM_DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
 							" op%d($v, <string>) -> $v\n",
 							rve->fpos.s_line, rve->fpos.s_col,
 							rve->fpos.e_line, rve->fpos.e_col, op);
@@ -3406,31 +3408,31 @@ static int rve_opt_01(struct rval_expr* rve, enum rval_type rve_type)
 			}else{
 				if (rve->op==RVE_RVAL_OP){
 					if (rve->left.rval.type==RV_STR)
-						DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
+						LM_DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
 								" op%d(<string>, $v) -> \"%s\"\n", 
 								rve->fpos.s_line, rve->fpos.s_col,
 								rve->fpos.e_line, rve->fpos.e_col,
 								op, rve->left.rval.v.s.s);
 					else
-						DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
+						LM_DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
 								" op%d(<string>, $v) -> $v (rval)\n",
 								rve->fpos.s_line, rve->fpos.s_col,
 								rve->fpos.e_line, rve->fpos.e_col, op);
 				}else if (rve->op==RVE_STR_OP){
 					if (rve->left.rve->op==RVE_RVAL_OP &&
 							rve->left.rve->left.rval.type==RV_STR)
-						DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
+						LM_DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
 								" op%d(<string>, $v) -> (str)\"%s\"\n", 
 								rve->fpos.s_line, rve->fpos.s_col,
 								rve->fpos.e_line, rve->fpos.e_col,
 								op, rve->left.rve->left.rval.v.s.s);
 					else
-						DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
+						LM_DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
 								" op%d(<string>, $v) -> (str)$v\n",
 								rve->fpos.s_line, rve->fpos.s_col,
 								rve->fpos.e_line, rve->fpos.e_col, op);
 				}else{
-					DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
+					LM_DBG("FIXUP RVE: (%d,%d-%d,%d) optimized"
 							" op%d(<string>, $v) -> $v\n",
 							rve->fpos.s_line, rve->fpos.s_col,
 							rve->fpos.e_line, rve->fpos.e_col, op);
@@ -3479,13 +3481,13 @@ static int rve_optimize(struct rval_expr* rve)
 		rv=0;
 		trv=&rve->left.rval;
 		if (trv->type==RV_INT)
-			DBG("FIXUP RVE (%d,%d-%d,%d): optimized constant int rve "
+			LM_DBG("FIXUP RVE (%d,%d-%d,%d): optimized constant int rve "
 					"(old op %d) to %d\n",
 					rve->fpos.s_line, rve->fpos.s_col,
 					rve->fpos.e_line, rve->fpos.e_col,
 					op, (int)trv->v.l);
 		else if (trv->type==RV_STR)
-			DBG("FIXUP RVE (%d,%d-%d,%d): optimized constant str rve "
+			LM_DBG("FIXUP RVE (%d,%d-%d,%d): optimized constant str rve "
 					"(old op %d) to \"%.*s\"\n",
 					rve->fpos.s_line, rve->fpos.s_col,
 					rve->fpos.e_line, rve->fpos.e_col,
@@ -3524,7 +3526,7 @@ static int rve_optimize(struct rval_expr* rve)
 				if (rve_replace_with_ct_rv(rve->right.rve, rv)<0)
 					goto error;
 				rve->op=RVE_IPLUS_OP;
-				DBG("FIXUP RVE (%d,%d-%d,%d): optimized $v - a into "
+				LM_DBG("FIXUP RVE (%d,%d-%d,%d): optimized $v - a into "
 						"$v + (%d)\n",
 						rve->fpos.s_line, rve->fpos.s_col,
 						rve->fpos.e_line, rve->fpos.e_col,
@@ -3539,12 +3541,12 @@ static int rve_optimize(struct rval_expr* rve)
 			l_type=rve_guess_type(rve->left.rve);
 			if (l_type==RV_INT){
 				rve->op=RVE_IPLUS_OP;
-				DBG("FIXUP RVE (%d,%d-%d,%d): changed + into integer plus\n",
+				LM_DBG("FIXUP RVE (%d,%d-%d,%d): changed + into integer plus\n",
 						rve->fpos.s_line, rve->fpos.s_col,
 						rve->fpos.e_line, rve->fpos.e_col);
 			}else if (l_type==RV_STR){
 				rve->op=RVE_CONCAT_OP;
-				DBG("FIXUP RVE (%d,%d-%d,%d): changed + into string concat\n",
+				LM_DBG("FIXUP RVE (%d,%d-%d,%d): changed + into string concat\n",
 						rve->fpos.s_line, rve->fpos.s_col,
 						rve->fpos.e_line, rve->fpos.e_col);
 			}
@@ -3555,13 +3557,13 @@ static int rve_optimize(struct rval_expr* rve)
 			l_type=rve_guess_type(rve->left.rve);
 			if (l_type==RV_INT){
 				rve->op=(rve->op==RVE_EQ_OP)?RVE_IEQ_OP:RVE_IDIFF_OP;
-				DBG("FIXUP RVE (%d,%d-%d,%d): changed ==/!= into integer"
+				LM_DBG("FIXUP RVE (%d,%d-%d,%d): changed ==/!= into integer"
 						" ==/!=\n",
 						rve->fpos.s_line, rve->fpos.s_col,
 						rve->fpos.e_line, rve->fpos.e_col);
 			}else if (l_type==RV_STR){
 				rve->op=(rve->op==RVE_EQ_OP)?RVE_STREQ_OP:RVE_STRDIFF_OP;
-				DBG("FIXUP RVE (%d,%d-%d,%d): changed ==/!= into string"
+				LM_DBG("FIXUP RVE (%d,%d-%d,%d): changed ==/!= into string"
 						" ==/!=\n",
 						rve->fpos.s_line, rve->fpos.s_col,
 						rve->fpos.e_line, rve->fpos.e_col);
@@ -3592,7 +3594,7 @@ static int rve_optimize(struct rval_expr* rve)
 					   or a combination of them */
 					if ((rve->op==RVE_PLUS_OP) &&
 						(rve_guess_type(tmp_rve.left.rve)!=RV_STR)){
-						DBG("RVE optimization failed (%d,%d-%d,%d): cannot "
+						LM_DBG("RVE optimization failed (%d,%d-%d,%d): cannot "
 							"optimize +(+($v, a), b) when typeof(a)==INT\n",
 							rve->fpos.s_line, rve->fpos.s_col,
 							rve->fpos.e_line, rve->fpos.e_col);
@@ -3611,13 +3613,13 @@ static int rve_optimize(struct rval_expr* rve)
 					rve->left.rve=rve->left.rve->left.rve;
 					trv=&rve->right.rve->left.rval;
 					if (trv->type==RV_INT)
-						DBG("FIXUP RVE (%d,%d-%d,%d): optimized int rve: "
+						LM_DBG("FIXUP RVE (%d,%d-%d,%d): optimized int rve: "
 								"op(op($v, a), b) with op($v, %d); op=%d\n",
 								rve->fpos.s_line, rve->fpos.s_col,
 								rve->fpos.e_line, rve->fpos.e_col,
 								(int)trv->v.l, rve->op);
 					else if (trv->type==RV_STR)
-						DBG("FIXUP RVE (%d,%d-%d,%d): optimized str rve "
+						LM_DBG("FIXUP RVE (%d,%d-%d,%d): optimized str rve "
 								"op(op($v, a), b) with op($v, \"%.*s\");"
 								" op=%d\n",
 								rve->fpos.s_line, rve->fpos.s_col,
@@ -3649,13 +3651,13 @@ static int rve_optimize(struct rval_expr* rve)
 					rv=0;
 					trv=&rve->left.rve->left.rval;
 					if (trv->type==RV_INT)
-						DBG("FIXUP RVE (%d,%d-%d,%d): optimized int rve: "
+						LM_DBG("FIXUP RVE (%d,%d-%d,%d): optimized int rve: "
 								"op(op(a, $v), b) with op(%d, $v); op=%d\n",
 								rve->fpos.s_line, rve->fpos.s_col,
 								rve->fpos.e_line, rve->fpos.e_col,
 								(int)trv->v.l, rve->op);
 					else if (trv->type==RV_STR)
-						DBG("FIXUP RVE (%d,%d-%d,%d): optimized str rve "
+						LM_DBG("FIXUP RVE (%d,%d-%d,%d): optimized str rve "
 								"op(op(a, $v), b) with op(\"%.*s\", $v);"
 								" op=%d\n",
 								rve->fpos.s_line, rve->fpos.s_col,
@@ -3695,13 +3697,13 @@ static int rve_optimize(struct rval_expr* rve)
 					rve->right.rve=rve->right.rve->left.rve;
 					trv=&rve->left.rve->left.rval;
 					if (trv->type==RV_INT)
-						DBG("FIXUP RVE (%d,%d-%d,%d): optimized int rve: "
+						LM_DBG("FIXUP RVE (%d,%d-%d,%d): optimized int rve: "
 								"op(a, op($v, b)) with op(%d, $v); op=%d\n",
 								rve->fpos.s_line, rve->fpos.s_col,
 								rve->fpos.e_line, rve->fpos.e_col,
 								(int)trv->v.l, rve->op);
 					else if (trv->type==RV_STR)
-						DBG("FIXUP RVE (%d,%d-%d,%d): optimized str rve "
+						LM_DBG("FIXUP RVE (%d,%d-%d,%d): optimized str rve "
 								"op(a, op($v, b)) with op(\"%.*s\", $v);"
 								" op=%d\n",
 								rve->fpos.s_line, rve->fpos.s_col,
@@ -3719,7 +3721,7 @@ static int rve_optimize(struct rval_expr* rve)
 					if ((rve->op==RVE_PLUS_OP) &&
 						(rve_guess_type(tmp_rve.left.rve) != 
 						 	rve_guess_type(tmp_rve.right.rve))){
-						DBG("RVE optimization failed (%d,%d-%d,%d): cannot "
+						LM_DBG("RVE optimization failed (%d,%d-%d,%d): cannot "
 								"optimize +(a, +(b, $v)) when "
 								"typeof(a)!=typeof(b)\n",
 								rve->fpos.s_line, rve->fpos.s_col,
@@ -3739,13 +3741,13 @@ static int rve_optimize(struct rval_expr* rve)
 					rve->right.rve=rve->right.rve->right.rve;
 					trv=&rve->left.rve->left.rval;
 					if (trv->type==RV_INT)
-						DBG("FIXUP RVE (%d,%d-%d,%d): optimized int rve: "
+						LM_DBG("FIXUP RVE (%d,%d-%d,%d): optimized int rve: "
 								"op(a, op(b, $v)) with op(%d, $v); op=%d\n",
 								rve->fpos.s_line, rve->fpos.s_col,
 								rve->fpos.e_line, rve->fpos.e_col,
 								(int)trv->v.l, rve->op);
 					else if (trv->type==RV_STR)
-						DBG("FIXUP RVE (%d,%d-%d,%d): optimized str rve "
+						LM_DBG("FIXUP RVE (%d,%d-%d,%d): optimized str rve "
 								"op(a, op(b, $v)) with op(\"%.*s\", $v);"
 								" op=%d\n",
 								rve->fpos.s_line, rve->fpos.s_col,
@@ -3796,6 +3798,7 @@ int fix_rval_expr(void* p)
 		case RVE_STRLEN_OP:
 		case RVE_STREMPTY_OP:
 		case RVE_DEFINED_OP:
+		case RVE_NOTDEFINED_OP:
 		case RVE_INT_OP:
 		case RVE_STR_OP:
 			ret=fix_rval_expr((void*)rve->left.rve);
